/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.generator.entity.grammardefnode;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import net.sf.parser4j.generator.service.GrammarDefNodeVisitException;
import net.sf.parser4j.generator.service.grammardefnode.IGrammarDefVisitor;
import net.sf.parser4j.kernelgenerator.service.GeneratorException;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class RulesDef implements IGrammarDefNode, IGrammarDefVisitor {

	private List<RuleDef> list = new LinkedList<RuleDef>();// NOPMD

	private Map<String, StatementDef> statementDefByNonTerminalNameMap = // NOPMD
	/**/new TreeMap<String, StatementDef>();

	private String whiteSpaceNonTerminalName;

	public RulesDef(final RuleDef ruleDef) throws GeneratorException {
		list.add(ruleDef);
		if (statementDefByNonTerminalNameMap.put(ruleDef
				.getNonTerminalDefinedName(), ruleDef.getStatementDef()) != null) {
			throw new GeneratorException("duplicate definition of "
					+ ruleDef.getNonTerminalDefinedName());
		}
	}

	public RulesDef(final RulesDef rulesDef, final RuleDef ruleDef)
			throws GeneratorException {
		list.addAll(rulesDef.list);
		statementDefByNonTerminalNameMap
				.putAll(rulesDef.statementDefByNonTerminalNameMap);
		list.add(ruleDef);
		if (statementDefByNonTerminalNameMap.put(ruleDef
				.getNonTerminalDefinedName(), ruleDef.getStatementDef()) != null) {
			throw new GeneratorException("duplicate definition of "
					+ ruleDef.getNonTerminalDefinedName());
		}
	}

	public void setWhiteSpaceNonTerminalName(
			final String whiteSpaceNonTerminalName) throws GeneratorException {
		this.whiteSpaceNonTerminalName = whiteSpaceNonTerminalName;
		try {
			accept(this);
		} catch (GrammarDefNodeVisitException exception) {
			throw new GeneratorException(exception);
		}
	}

	public String getWhiteSpaceNonTerminalName() {
		return whiteSpaceNonTerminalName;
	}

	public void addAll(final RulesDef toAdd) {
		list.addAll(toAdd.list);
	}

	public List<RuleDef> getList() {
		return list;
	}

	/**
	 * @return the statementDefByNonTerminalNameMap
	 */
	public Map<String, StatementDef> getStatementDefByNonTerminalNameMap() {
		return statementDefByNonTerminalNameMap;
	}

	public void accept(final IGrammarDefVisitor visitor)
			throws GrammarDefNodeVisitException {
		visitor.beginVisit(this);
		for (RuleDef ruleDef : list) {
			ruleDef.accept(visitor);
		}
		visitor.endVisit(this);
	}

	@Override
	public void beginVisit(final GrammarDef def)
			throws GrammarDefNodeVisitException {
		unsupported();
	}

	@Override
	public void endVisit(final GrammarDef def)
			throws GrammarDefNodeVisitException {
		unsupported();
	}

	@Override
	public void beginVisit(final RulesGroupListDef def)
			throws GrammarDefNodeVisitException {
		unsupported();
	}

	@Override
	public void endVisit(final RulesGroupListDef def)
			throws GrammarDefNodeVisitException {
		unsupported();
	}

	private void unsupported() {
		throw new UnsupportedOperationException("must not be visited");
	}

	@Override
	public void beginVisit(final RulesDef def)
			throws GrammarDefNodeVisitException {
		// nothing to implement
	}

	@Override
	public void endVisit(final RulesDef def)
			throws GrammarDefNodeVisitException {
		// nothing to implement
	}

	@Override
	public void beginVisit(final RuleDef def)
			throws GrammarDefNodeVisitException {
		// nothing to implement
	}

	@Override
	public void endVisit(final RuleDef def) throws GrammarDefNodeVisitException {
		// nothing to implement
	}

	@Override
	public boolean beginVisit(final StatementDef def)
			throws GrammarDefNodeVisitException {
		def.setWhiteSpaceNonTerminalName(whiteSpaceNonTerminalName);
		return true;
	}

	@Override
	public void endVisit(final StatementDef def)
			throws GrammarDefNodeVisitException {
		// nothing to implement
	}

	@Override
	public String toString() {
		return "RulesDef";
	}
}
