/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.generator.entity.grammardefnode;

import java.util.Map;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class RulesGroupDef extends AbstractGrammarDefNodeNotVisitable {

	private static final String EMPTY_STRING = "";

	/** white space non terminal name for group of rules */
	private final String whiteSpaceNonTerminalName;// NOPMD

	private final RulesDef rulesDef;// NOPMD

	public RulesGroupDef(final String whiteSpaceNonTerminalName,
			final RulesDef rulesDef) {
		super();
		this.whiteSpaceNonTerminalName = whiteSpaceNonTerminalName;
		this.rulesDef = rulesDef;
	}

	public RulesGroupDef(final RulesDef rulesDef) {
		super();
		this.whiteSpaceNonTerminalName = EMPTY_STRING;
		this.rulesDef = rulesDef;
	}

	public String getWhiteSpaceNonTerminalName() {
		return whiteSpaceNonTerminalName;
	}

	public RulesDef getRulesDef() {
		return rulesDef;
	}

	/**
	 * @return
	 * @see net.sf.parser4j.generator.entity.grammardefnode.RulesDef#getStatementDefByNonTerminalNameMap()
	 */
	public Map<String, StatementDef> getStatementDefByNonTerminalNameMap() {
		return rulesDef.getStatementDefByNonTerminalNameMap();
	}

	@Override
	public String toString() {
		final StringBuilder stringBuilder = new StringBuilder();
		stringBuilder.append("RulesGroupDef, white space non terminal: ");
		stringBuilder.append(whiteSpaceNonTerminalName);
		return stringBuilder.toString();
	}
}
