/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.generator.entity.grammardefnode;

import java.util.Map;
import java.util.TreeMap;

import net.sf.parser4j.generator.service.GrammarDefNodeVisitException;
import net.sf.parser4j.generator.service.grammardefnode.IGrammarDefVisitor;
import net.sf.parser4j.kernelgenerator.service.GeneratorException;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class RulesGroupListDef implements IGrammarDefNode {

	private Map<String, RulesDef> rulesDefByWhiteSpaceNameMap = // NOPMD
	/**/new TreeMap<String, RulesDef>();

	private Map<String, StatementDef> statementDefByNonTerminalNameMap = // NOPMD
	/**/new TreeMap<String, StatementDef>();

	/**
	 * construct an empty rules group
	 */
	public RulesGroupListDef() {
		super();
	}

	public RulesGroupListDef(final RulesGroupListDef rulesGroupListDef,
			final RulesGroupDef rulesGroupDef) throws GeneratorException {
		super();
		rulesDefByWhiteSpaceNameMap
				.putAll(rulesGroupListDef.rulesDefByWhiteSpaceNameMap);
		final String whiteSpaceNonTerminalName = rulesGroupDef
				.getWhiteSpaceNonTerminalName();
		final RulesDef existing = rulesDefByWhiteSpaceNameMap
				.get(whiteSpaceNonTerminalName);
		final RulesDef toAdd = rulesGroupDef.getRulesDef();
		toAdd.setWhiteSpaceNonTerminalName(whiteSpaceNonTerminalName);
		if (existing == null) {
			rulesDefByWhiteSpaceNameMap.put(whiteSpaceNonTerminalName, toAdd);
		} else {
			existing.addAll(toAdd);
		}
		statementDefByNonTerminalNameMap
				.putAll(rulesGroupListDef.statementDefByNonTerminalNameMap);
		for (Map.Entry<String, StatementDef> entry : rulesGroupDef
				.getStatementDefByNonTerminalNameMap().entrySet()) {
			if (statementDefByNonTerminalNameMap.put(entry.getKey(), entry
					.getValue()) != null) {
				throw new GeneratorException("duplicate definition of "
						+ entry.getKey());
			}
		}
	}

	public RulesGroupListDef(final RulesGroupDef rulesGroupDef)
			throws GeneratorException {
		super();
		final RulesDef toAdd = rulesGroupDef.getRulesDef();
		final String whiteSpaceNonTerminalName = rulesGroupDef
				.getWhiteSpaceNonTerminalName();
		toAdd.setWhiteSpaceNonTerminalName(whiteSpaceNonTerminalName);
		rulesDefByWhiteSpaceNameMap.put(whiteSpaceNonTerminalName, toAdd);
		statementDefByNonTerminalNameMap.putAll(rulesGroupDef
				.getStatementDefByNonTerminalNameMap());
	}

	public Map<String, RulesDef> getRulesDefByWhiteSpaceNameMap() {
		return rulesDefByWhiteSpaceNameMap;
	}

	/**
	 * @return the statementDefByNonTerminalNameMap
	 */
	public Map<String, StatementDef> getStatementDefByNonTerminalNameMap() {
		return statementDefByNonTerminalNameMap;
	}

	public void accept(final IGrammarDefVisitor visitor)
			throws GrammarDefNodeVisitException {
		visitor.beginVisit(this);
		for (RulesDef rulesDef : rulesDefByWhiteSpaceNameMap.values()) {
			rulesDef.accept(visitor);
		}
		visitor.endVisit(this);
	}

	@Override
	public String toString() {
		return "RulesGroupListDef";
	}
}
