/*
 * Copyright 2009 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.generator.service;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.util.Map;
import java.util.TreeMap;

import org.apache.log4j.Logger;

import net.sf.parser4j.kernelgenerator.entity.grammarnode.IGrammarNode;
import net.sf.parser4j.kernelgenerator.service.GeneratorException;
import net.sf.parser4j.kernelgenerator.service.GrammarNodeVisitException;
import net.sf.parser4j.kernelgenerator.service.grammarnode.AbstractGrammarNodeVisitor;

/**
 * output grammar to text file
 * 
 * @author luc peuvrier
 * 
 */
public final class GrammarToTextFile extends AbstractGrammarNodeVisitor {

	private static final Logger _log = Logger
			.getLogger(GrammarToTextFile.class);

	private final static GrammarToTextFile INSTANCE =
	/**/new GrammarToTextFile();

	private transient Map<String, String> toStringMap;

	public static GrammarToTextFile getInstance() {
		return INSTANCE;
	}

	private GrammarToTextFile() {
		super();
	}

	public void generateTextFile(final IGrammarNode rootGrammarNode,
			final File textFile) throws GeneratorException {

		initialize();
		toStringMap = new TreeMap<String, String>();
		try {
			rootGrammarNode.accept(this);
		} catch (GrammarNodeVisitException exception) {
			throw new GeneratorException(exception);
		}
		close();

		FileOutputStream fileOutputStream;
		try {
			fileOutputStream = new FileOutputStream(textFile);
		} catch (FileNotFoundException exception) {
			throw new GeneratorException(exception);
		}
		OutputStreamWriter outputStreamWriter;
		try {
			outputStreamWriter = new OutputStreamWriter(fileOutputStream,
					"UTF8");
		} catch (UnsupportedEncodingException exception) {
			try {
				fileOutputStream.close();
			} catch (IOException exception2) {
				_log.error("closing after exception", exception2);
			}
			throw new GeneratorException(exception);
		}

		final BufferedWriter bufferedWriter = new BufferedWriter(
				outputStreamWriter);
		try {
			for (String name : toStringMap.keySet()) {
				bufferedWriter.write(toStringMap.get(name));
				bufferedWriter.write('\n');
			}
		} catch (IOException exception) {
			throw new GeneratorException(exception);
		} finally {
			try {
				bufferedWriter.close();
			} catch (IOException exception) {
				_log.error("closing after exception", exception);
			}
		}
		toStringMap = null;// NOPMD free memory
	}

	@Override
	protected void beginVisitImpl(final IGrammarNode grammarNode)
			throws GrammarNodeVisitException {
		toStringMap.put(grammarNode.getNonTerminalName(), grammarNode
				.toDefinitionString());
	}

	@Override
	protected void endVisitImpl(final IGrammarNode grammarNode)
			throws GrammarNodeVisitException {
		// no implementation
	}

	@Override
	protected void visitImpl(final IGrammarNode grammarNode)
			throws GrammarNodeVisitException {
		// no implementation
	}
}
