/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.generator.service.match;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;

import net.sf.parser4j.generator.entity.grammardefnode.RulesGroupListDef;
import net.sf.parser4j.generator.entity.grammardefnode.StatementDef;
import net.sf.parser4j.parser.entity.IParseSessionForMatchMgr;
import net.sf.parser4j.parser.entity.parsenode.IParseNode;
import net.sf.parser4j.parser.service.ParserException;
import net.sf.parser4j.parser.service.ParserFileReader;
import net.sf.parser4j.parser.service.match.AbstractMatchMgrAdapter;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class MatchInclude extends AbstractMatchMgrAdapter {

	@Override
	public void reduceAction(final IParseSessionForMatchMgr parseSession,
			final IParseNode fatherParseNode, final IParseNode[] parseNodes,
			final int[] notWhiteIndexes) throws ParserException {
		final String parserInputFileName = parseSession
				.getParserInputFileName();
		final String parent = new File(parserInputFileName).getParent();
		final StatementDef data = (StatementDef) parseNodes[notWhiteIndexes[1]]
				.getUniqData();
		final String includeParam = data.getTerminalString();
		final String toIncludeFileName;
		if (parent == null) {
			toIncludeFileName = includeParam;
		} else {
			toIncludeFileName = parent + File.separator + includeParam;
		}
		final InputStream inputStream;
		try {
			inputStream = new FileInputStream(toIncludeFileName);
		} catch (FileNotFoundException exception) {
			throw new ParserException(
					"creating reader of " + toIncludeFileName, exception);
		}
		final InputStreamReader inputStreamReader;
		try {
			inputStreamReader = new InputStreamReader(inputStream, "UTF-8");
		} catch (UnsupportedEncodingException exception) {
			throw new ParserException(
					"creating reader of " + toIncludeFileName, exception);
		}
		final ParserFileReader parserFileReader = new ParserFileReader(
				inputStreamReader, toIncludeFileName);
		parseSession.include(parserFileReader);
		final RulesGroupListDef resultRulesGroupListDef = new RulesGroupListDef();
		fatherParseNode.setData(resultRulesGroupListDef);
	}
}
