/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.entity;

import java.util.Iterator;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.TreeMap;
import java.util.Map.Entry;

import net.sf.parser4j.parser.entity.data.TerminalCharRange;
import net.sf.parser4j.parser.service.ExStringBuilder;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class GotoByTerminalMap {

	private final Map<TerminalCharRange, State> gotoMap = // NOPMD
	/**/new TreeMap<TerminalCharRange, State>();

	private final State fromState; // NOPMD

	public GotoByTerminalMap(final State fromState) {
		this.fromState = fromState;
	}

	public void addGoto(final TerminalCharRange terminalCharRange,
			final State state) {
		if (gotoMap.put(terminalCharRange, state) != null) {
			final ExStringBuilder builder = new ExStringBuilder();
			builder.append("\ngoto from state:\n");
			builder.appendTabulate(1, fromState.toString());
			builder.append("to state:\n");
			builder.appendTabulate(1, state.toString());
			builder.append("\nby terminal char range ");
			builder.append(terminalCharRange.toString());
			builder.append("\nalready in map");
			throw new IllegalArgumentException(builder.toString());
		}
	}

	public Iterator<StateByTerminalRange> iterator() {
		return new Iterator<StateByTerminalRange>() {

			private final Iterator<Map.Entry<TerminalCharRange, State>> iterator = // NOPMD
			/**/gotoMap.entrySet().iterator();

			public boolean hasNext() {
				return iterator.hasNext();
			}

			public StateByTerminalRange next() {
				if (!hasNext()) {
					throw new NoSuchElementException();
				}
				final Entry<TerminalCharRange, State> entry = iterator.next();
				return new StateByTerminalRange(entry.getKey(), entry
						.getValue());
			}

			public void remove() {
				throw new UnsupportedOperationException();
			}
		};
	}

}
