/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.entity;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.NavigableMap;
import java.util.Set;
import java.util.TreeMap;
import java.util.Map.Entry;

import net.sf.parser4j.parser.entity.data.TerminalCharRange;

/**
 * recognition point by terminal
 * 
 * @author luc peuvrier
 * 
 */
public class RecognitionPointByTerminalMap {

	/**
	 * 
	 */
	private final NavigableMap<Integer, RecognitionPointForTerminalRange>
	/**/lowerEndMap = // NOPMD
	/*  */new TreeMap<Integer, RecognitionPointForTerminalRange>();

	public void add(final TerminalCharRange terminalCharRange,
			final RecognitionPoint recognitionPoint) {
		final int firstTerminalValue =
		/**/terminalCharRange.getFirstTerminalValue();
		final int lastTerminalValue =
		/**/terminalCharRange.getLastTerminalValue();

		final List<RecognitionPointForTerminalRange> list = ranges(
				firstTerminalValue, lastTerminalValue);

		if (list.isEmpty()) {
			createRange(firstTerminalValue, lastTerminalValue, null,
					recognitionPoint);
		} else {
			RecognitionPointForTerminalRange previous = null;
			for (RecognitionPointForTerminalRange range : list) {
				if (previous == null) {
					if (firstTerminalValue < range.getFirstTerminalValue()) {
						createRange(firstTerminalValue, range
								.getFirstTerminalValue() - 1, null,
								recognitionPoint);
					}
				} else if (previous.getLastTerminalValue() + 1 != range
						.getFirstTerminalValue()) {
					createRange(previous.getLastTerminalValue() + 1, range
							.getFirstTerminalValue() - 1, null,
							recognitionPoint);
				}
				addRecognitionPointToRange(firstTerminalValue,
						lastTerminalValue, range, recognitionPoint);
				previous = range;
			}
			if (lastTerminalValue > previous.getLastTerminalValue()) {
				createRange(previous.getLastTerminalValue() + 1,
						lastTerminalValue, null, recognitionPoint);
			}
		}
	}

	/**
	 * 
	 * @param firstTerminalValue
	 *            interval first value
	 * @param lastTerminalValue
	 *            interval last value
	 * @param range
	 *            existing range
	 * @param recognitionPoint
	 *            recognition point for interval [ firstTerminalValue ,
	 *            lastTerminalValue ]
	 */
	private void addRecognitionPointToRange(final int firstTerminalValue,
			final int lastTerminalValue,
			final RecognitionPointForTerminalRange range,
			final RecognitionPoint recognitionPoint) {
		if (firstTerminalValue <= range.getFirstTerminalValue()) {
			if (lastTerminalValue >= range.getLastTerminalValue()) {
				// ...first ........last...
				// ...|.............|......
				// .....===========........ range
				// .....range..............
				range.addRecognitionPoint(recognitionPoint);
			} else if (lastTerminalValue >= range.getFirstTerminalValue()) {
				// ...first .......last...
				// ...|............|......
				// ..........===========.. range
				// ..........11111112222.. new ranges 1111111 and 2222
				lowerEndMap.remove(range.getFirstTerminalValue());
				final Set<RecognitionPoint> setOfRecognitionPoint = range
						.getSetOfRecognitionPoint();
				// create new range 1111111
				createRange(range.getFirstTerminalValue(), lastTerminalValue,
						setOfRecognitionPoint, recognitionPoint);
				// create new range 2222
				createRange(lastTerminalValue + 1,
						range.getLastTerminalValue(), setOfRecognitionPoint,
						null);
			}// else
			// ...first ....last...........
			// ...|.........|..............
			// ...............===========.. range
		} else {
			final Set<RecognitionPoint> setOfRecognitionPoint = range
					.getSetOfRecognitionPoint();
			if (firstTerminalValue <= range.getLastTerminalValue()) {
				if (lastTerminalValue >= range.getLastTerminalValue()) {
					// ..... first .... last ....
					// ......|..........|........
					// ...=========.............. range
					// ...111222222.............. new ranges 111 and 222222
					lowerEndMap.remove(range.getFirstTerminalValue());
					// create new range 111
					createRange(range.getFirstTerminalValue(),
							firstTerminalValue - 1, setOfRecognitionPoint, null);
					// create new range 222222
					createRange(firstTerminalValue, range
							.getLastTerminalValue(), setOfRecognitionPoint,
							recognitionPoint);
				} else {
					// ..... first .... last ....
					// ......|......... | .......
					// ...=================...... range
					// ...11122222222222233...... new range 111, 2..2, 33
					lowerEndMap.remove(range.getFirstTerminalValue());
					// create range 111
					createRange(range.getFirstTerminalValue(),
							firstTerminalValue - 1, setOfRecognitionPoint, null);
					// create range 2..2
					createRange(firstTerminalValue, lastTerminalValue,
							setOfRecognitionPoint, recognitionPoint);
					// create range 33
					createRange(lastTerminalValue + 1, range
							.getLastTerminalValue(), setOfRecognitionPoint,
							null);
				}
			}// else
			// .............. first .... last ....
			// ...............|..........|........
			// ...=========....................... range
		}
	}

	/**
	 * create a new range
	 * 
	 * @param newFirst
	 *            new range first value
	 * @param newLast
	 *            new range last value
	 * @param setOfRecognitionPoint
	 *            set of recognition point of the new range, null if none
	 * @param recognitionPoint
	 *            a recognition point of the new range, null if none
	 */
	private void createRange(final int newFirst, final int newLast,
			final Set<RecognitionPoint> setOfRecognitionPoint,
			final RecognitionPoint recognitionPoint) {
		final TerminalCharRange newTerminalRange = new TerminalCharRange(
				newFirst, newLast);
		final RecognitionPointForTerminalRange newRange = new RecognitionPointForTerminalRange(
				newTerminalRange);
		if (setOfRecognitionPoint != null) {
			for (RecognitionPoint element : setOfRecognitionPoint) {
				newRange.addRecognitionPoint(element);
			}
		}
		if (recognitionPoint != null) {
			newRange.addRecognitionPoint(recognitionPoint);
		}
		lowerEndMap.put(newFirst, newRange);
	}

	private List<RecognitionPointForTerminalRange> ranges(
			final int firstTerminalValue, final int lastTerminalValue) {
		final List<RecognitionPointForTerminalRange> list =
		/**/new LinkedList<RecognitionPointForTerminalRange>();
		RecognitionPointForTerminalRange previous = null;
		RecognitionPointForTerminalRange current =
		/**/firstRange(firstTerminalValue);
		if (current != null
				&& current.getFirstTerminalValue() > lastTerminalValue) {
			current = null;// NOPMD
		}
		while (current != null) {
			list.add(current);
			previous = current;
			final Entry<Integer, RecognitionPointForTerminalRange> ceilingEntry =
			/**/lowerEndMap.ceilingEntry(previous.getLastTerminalValue() + 1);
			if (ceilingEntry == null) {
				current = null;// NOPMD
			} else {
				current = ceilingEntry.getValue();
				if (current != null
						&& current.getFirstTerminalValue() > lastTerminalValue) {
					current = null;// NOPMD
				}
			}
		}
		return list;
	}

	private RecognitionPointForTerminalRange firstRange(
			final int firstTerminalValue) {
		final Entry<Integer, RecognitionPointForTerminalRange> floorEntry =
		/**/lowerEndMap.floorEntry(firstTerminalValue);
		final RecognitionPointForTerminalRange firstRange;
		if (floorEntry == null
				|| floorEntry.getValue().getLastTerminalValue() < firstTerminalValue) {
			final Entry<Integer, RecognitionPointForTerminalRange> ceilingEntry =
			/**/lowerEndMap.ceilingEntry(firstTerminalValue);
			if (ceilingEntry == null) {
				firstRange = null;
			} else {
				firstRange = ceilingEntry.getValue();
			}
		} else {
			firstRange = floorEntry.getValue();
		}
		return firstRange;
	}

	public Iterator<RecognitionPointForTerminalRange> iterator() {
		return lowerEndMap.values().iterator();
	}
}
