/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.entity;

import java.util.Map;
import java.util.TreeMap;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class StateMap {

	private int nextStateIdentifier = 0;// NOPMD

	private final Map<State, State> existingStateMap = // NOPMD
	/**/new TreeMap<State, State>();

	private final Map<Integer, State> stateByIdentifierMap = // NOPMD
	/**/new TreeMap<Integer, State>();

	/**
	 * 
	 * @param state
	 * @return existing state for state
	 */
	public State existingState(final State state) {
		return existingStateMap.get(state);
	}

	/**
	 * add state<br>
	 * if state exist return false, else add new state and set its state
	 * identifier number<br>
	 * 
	 * @param state
	 *            state to add, can already exist
	 */
	public void addState(final State state) {
		State existing = existingStateMap.get(state);
		if (existing == null) {
			existing = state;
			state.setStateIdentifier(nextStateIdentifier++);
			existingStateMap.put(state, state);
			stateByIdentifierMap.put(state.getStateIdentifier(), state);
		} else {
			throw new IllegalArgumentException("state exist");
		}
	}

	/**
	 * 
	 * @param stateIdentifier
	 * @return existing state for state identifier
	 */
	public State stateByIdentifier(final int stateIdentifier) {
		return stateByIdentifierMap.get(stateIdentifier);
	}

	public int numberOfState() {
		return nextStateIdentifier;
	}
}
