/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.entity.grammarnode;

import net.sf.parser4j.parser.entity.EnumNodeType;

/**
 * 
 * @author luc peuvrier
 * 
 */
public abstract class AbstractGrammarNode implements IGrammarNode {

	/** grammar node type */
	private final EnumNodeType grammarNodeType;// NOPMD

	/** non terminal identifier for this node */
	private int nonTerminalIdentifier;

	/** non terminal name */
	protected String nonTerminalName;

	protected String matchClassSimpleName;

	private boolean token = false;// value by default

	protected AbstractGrammarNode(final EnumNodeType grammarNodeType) {
		super();
		this.grammarNodeType = grammarNodeType;
	}

	public EnumNodeType getGrammarNodeType() {
		return grammarNodeType;
	}

	public void setNonTerminalIdentifier(final int nonTerminalIdentifier) {
		this.nonTerminalIdentifier = nonTerminalIdentifier;
	}

	public void setNonTerminalName(final String nonTerminalName) {
		this.nonTerminalName = nonTerminalName;
	}

	public int getNonTerminalIdentifier() {
		return nonTerminalIdentifier;
	}

	public String getNonTerminalName() {
		return nonTerminalName;
	}

	@Override
	public String getMatchClassSimpleName() {
		return matchClassSimpleName;
	}

	@Override
	public void setMatchClassSimpleName(final String matchClassSimpleName) {
		this.matchClassSimpleName = matchClassSimpleName;
	}

	public void setToken(final boolean token) {
		this.token = token;
	}

	@Override
	public boolean isToken() {
		return token;
	}

	public boolean isTerminal() {
		return grammarNodeType.isTerminal();
	}

	public boolean isNonTerminal() {
		return grammarNodeType.isNonTerminal();
	}

	@Override
	public boolean isStringTerminal() {
		return EnumNodeType.STRING_TERMINAL.equals(grammarNodeType);
	}

	@Override
	public boolean isEmptyType() {
		return EnumNodeType.EMPTY.equals(grammarNodeType);
	}

	/**
	 * throws IllegalArgumentException because of bad recognition point index
	 * value
	 * 
	 * @param recognitionPointIndex
	 * @throws IllegalArgumentException
	 *             bad recognition point index value
	 */
	protected void badRecognitionPointIndex(final int recognitionPointIndex) {
		throw new IllegalArgumentException("bad recognition point index "
				+ recognitionPointIndex);
	}

	public int compareTo(final IGrammarNode grammarNode) {
		final int compareTo;
		final int otherNonTerminalIdentifier = grammarNode
				.getNonTerminalIdentifier();
		if (nonTerminalIdentifier > otherNonTerminalIdentifier) {
			compareTo = 1;
		} else if (nonTerminalIdentifier < otherNonTerminalIdentifier) {
			compareTo = -1;
		} else {
			compareTo = 0;
		}
		return compareTo;
	}

	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + nonTerminalIdentifier;
		return result;
	}

	@Override
	@SuppressWarnings("PMD")
	public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final AbstractGrammarNode other = (AbstractGrammarNode) obj;
		if (nonTerminalIdentifier != other.getNonTerminalIdentifier())
			return false;
		return true;
	}

	protected String characterValueToString(final int value) {
		final StringBuilder builder = new StringBuilder();
		if (value >= ' ' && value < 0x7f) {
			builder.append('\'');
			builder.append((char) value);
			builder.append('\'');
		} else {
			builder.append("\\x");
			builder.append(String.format("%02X", new Object[] { value }));
		}
		return builder.toString();
	}

	protected String nonTerminalToString() {
		final String nonTerminalToString;
		if (token) {
			final StringBuilder stringBuilder = new StringBuilder();
			stringBuilder.append('[');
			stringBuilder.append(nonTerminalName);
			stringBuilder.append(']');
			nonTerminalToString = stringBuilder.toString();
		} else {
			nonTerminalToString = nonTerminalName;
		}
		return nonTerminalToString;
	}

	protected String matchClassToString() {
		final String matchClassToString;
		if (matchClassSimpleName == null) {
			matchClassToString = "";
		} else {
			final StringBuilder builder = new StringBuilder();
			builder.append("\n { ");
			builder.append(matchClassSimpleName);
			builder.append(" }");
			matchClassToString = builder.toString();
		}
		return matchClassToString;
	}

	@Override
	public String toString() {
		final StringBuilder builder = new StringBuilder();
		builder.append("NT ");
		builder.append(grammarNodeType);
		builder.append(" #");
		builder.append(nonTerminalIdentifier);
		if (nonTerminalName != null) {
			builder.append(" '");
			builder.append(nonTerminalName);
			builder.append('\'');
		}
		builder.append(" token=");
		builder.append(token);
		return builder.toString();
	}

}
