/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.entity.grammarnode;

import java.util.ArrayList;
import java.util.List;

import net.sf.parser4j.kernelgenerator.entity.grammarnode.impl.OperandGrammarNode;
import net.sf.parser4j.kernelgenerator.service.GrammarNodeVisitException;
import net.sf.parser4j.kernelgenerator.service.grammarnode.IGrammarNodeVisitor;
import net.sf.parser4j.parser.entity.EnumNodeType;

/**
 * 
 * @author luc peuvrier
 * 
 */
public abstract class AbstractMularyGrammarNode extends AbstractNonTerminalNode {

	protected List<OperandGrammarNode> operandList;// NOPMD

	protected AbstractMularyGrammarNode(final EnumNodeType grammarNodeType) {
		super(grammarNodeType);
	}

	public void setOperandList(final List<OperandGrammarNode> operandList) {
		this.operandList = new ArrayList<OperandGrammarNode>(operandList.size());
		this.operandList.addAll(operandList);
	}

	public List<OperandGrammarNode> getOperandList() {
		return operandList;
	}

	@Override
	public int[] getRightPart() {
		int[] rightPart = new int[operandList.size()];
		int index = 0;
		for (OperandGrammarNode operandGrammarNode : operandList) {
			rightPart[index++] = operandGrammarNode.getNonTerminalIdentifier();
		}
		return rightPart;
	}

	public void accept(final IGrammarNodeVisitor grammarNodeVisitor)
			throws GrammarNodeVisitException {
		assertOperandListSetted();
		if (grammarNodeVisitor.beginVisit(this)) {
			for (OperandGrammarNode grammarNode : operandList) {
				grammarNode.accept(grammarNodeVisitor);
			}
			grammarNodeVisitor.endVisit(this);
		}
	}

	protected void assertOperandListSetted() {
		if (operandList == null) {
			throw new IllegalStateException("no operand list setted "
					+ toString());
		}
	}
}
