/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.entity.grammarnode;

import java.util.List;

import net.sf.parser4j.kernelgenerator.entity.grammarnode.impl.OperandGrammarNode;
import net.sf.parser4j.kernelgenerator.service.GrammarNodeVisitException;
import net.sf.parser4j.kernelgenerator.service.grammarnode.IGrammarNodeVisitor;
import net.sf.parser4j.parser.entity.EnumNodeType;

/**
 * common to all terminal grammar node
 * 
 * @author luc peuvrier
 * 
 */
public abstract class AbstractTerminalNode extends AbstractGrammarNode {

	private static final int[] RIGHT_PART = new int[] { -1 };

	protected AbstractTerminalNode() {
		super(EnumNodeType.CHARACTER_TERMINAL);
	}

	@Override
	public void setOperandList(final List<OperandGrammarNode> operandList) {
		throw new UnsupportedOperationException(getClass() + " have not son");
	}

	public boolean match(final int recognitionPointIndex) {
		final boolean match;
		if (recognitionPointIndex == 0) {
			match = false;
		} else if (recognitionPointIndex == 1) {
			match = true;
		} else {
			badRecognitionPointIndex(recognitionPointIndex);
			match = false;// for compiler warning
		}
		return match;
	}

	@Override
	public int[] getRightPart() {
		return RIGHT_PART;// NOPMD
	}

	public void accept(final IGrammarNodeVisitor grammarNodeVisitor)
			throws GrammarNodeVisitException {
		if (grammarNodeVisitor.beginVisit(this)) {
			grammarNodeVisitor.endVisit(this);
		}
	}

	public List<OperandGrammarNode> expectedNonTerminal(
			final int recognitionPointIndex) {
		throw new UnsupportedOperationException("is a terminal");
	}

	public abstract String valueToString();

}
