/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.entity.grammarnode;

import java.util.Iterator;
import java.util.List;

import net.sf.parser4j.kernelgenerator.entity.grammarnode.impl.OperandGrammarNode;
import net.sf.parser4j.kernelgenerator.service.grammarnode.IGrammarNodeVisitor;
import net.sf.parser4j.parser.entity.EnumNodeType;
import net.sf.parser4j.parser.entity.data.TerminalCharRange;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class GrammarNodeNull extends AbstractGrammarNode {

	final private String sonNonTerminalName;// NOPMD

	public GrammarNodeNull(final String sonNonTerminalName) {
		super(EnumNodeType.NULL);
		this.sonNonTerminalName = sonNonTerminalName;
	}

	public String getSonNonTerminalName() {
		return sonNonTerminalName;
	}

	@Override
	public void setOperandList(final List<OperandGrammarNode> operandList) {
		unsupported();
	}

	@Override
	public void accept(final IGrammarNodeVisitor grammarNodeVisitor) {
		unsupported();
	}

	private void unsupported() {
		throw new UnsupportedOperationException("null grammar node "
				+ getNonTerminalName());
	}

	@Override
	public List<OperandGrammarNode> expectedNonTerminal(
			final int recognitionPointIndex) {
		unsupported();
		return null;
	}

	@Override
	public boolean isTerminal() {
		unsupported();
		return false;
	}

	@Override
	public void setToken(final boolean token) {
		// no implementation
	}

	@Override
	public boolean isToken() {
		unsupported();
		return false;
	}

	@Override
	public boolean match(final int recognitionPointIndex) {
		unsupported();
		return false;
	}

	@Override
	public Iterator<TerminalCharRange> terminalValuesIterator() {
		unsupported();
		return null;
	}

	@Override
	protected String nonTerminalToString() {
		return nonTerminalName;
	}

	@Override
	public int[] getRightPart() {// NOPMD
		unsupported();
		return null;
	}

	@Override
	public String toDefinitionString() {
		return nonTerminalName + " : null " + matchClassToString();
	}

	@Override
	public String toString() {
		return super.toString() + " : null" + matchClassToString();
	}
}
