/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.entity.grammarnode;

import java.util.Iterator;
import java.util.List;

import net.sf.parser4j.kernelgenerator.entity.grammarnode.impl.OperandGrammarNode;
import net.sf.parser4j.kernelgenerator.service.GrammarNodeVisitException;
import net.sf.parser4j.kernelgenerator.service.grammarnode.IGrammarNodeVisitor;
import net.sf.parser4j.parser.entity.EnumNodeType;
import net.sf.parser4j.parser.entity.data.TerminalCharRange;

/**
 * grammar node interface
 * 
 * @author luc peuvrier
 * 
 */
public interface IGrammarNode extends Comparable<IGrammarNode> {

	/**
	 * 
	 * @return non terminal identifier
	 */
	int getNonTerminalIdentifier();

	/**
	 * 
	 * @return non terminal name
	 */
	String getNonTerminalName();

	/**
	 * 
	 * @param nonTerminalIdentifier
	 *            non terminal identifier
	 */
	void setNonTerminalIdentifier(int nonTerminalIdentifier);

	/**
	 * 
	 * @param nonTerminalName
	 *            non terminal name
	 */
	void setNonTerminalName(String nonTerminalName);

	void setOperandList(List<OperandGrammarNode> operandList);

	/**
	 * 
	 * @return grammar node type
	 */
	EnumNodeType getGrammarNodeType();

	/**
	 * 
	 * @return true is this grammar node is a terminal
	 */
	boolean isTerminal();

	boolean isNonTerminal();

	boolean isStringTerminal();

	boolean isEmptyType();

	void setToken(boolean token);

	/**
	 * 
	 * @return true if this grammar node define a token
	 */
	boolean isToken();

	/**
	 * 
	 * @return iterator on terminal values
	 */
	Iterator<TerminalCharRange> terminalValuesIterator();

	/**
	 * 
	 * @param matchClassSimpleName
	 *            match manager class simple name
	 */
	void setMatchClassSimpleName(String matchClassSimpleName);

	/**
	 * 
	 * @return match manager class simple name
	 */
	String getMatchClassSimpleName();

	/**
	 * 
	 * @param recognitionPointIndex
	 * @return true if match for recognition point index value
	 * @throws IllegalArgumentException
	 *             bad recognition point index value
	 */
	boolean match(int recognitionPointIndex);

	/**
	 * 
	 * @param recognitionPointIndex
	 * @return list of expected grammar node for a recognition point index
	 * @throws IllegalArgumentException
	 *             bad recognition point index value
	 */
	List<OperandGrammarNode> expectedNonTerminal(int recognitionPointIndex);

	/**
	 * 
	 * @return right part of the rule represented by this grammar node
	 */
	int[] getRightPart();

	/**
	 * visit by a visitor
	 * 
	 * @param grammarNodeVisitor
	 * @throws GrammarNodeVisitException
	 */
	void accept(IGrammarNodeVisitor grammarNodeVisitor)
			throws GrammarNodeVisitException;

	/**
	 * Returns a string full representation of the object.
	 * 
	 * @return
	 */
	String toString();

	String toDefinitionString();

}
