/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.entity.grammarnode.impl;

import java.util.ArrayList;
import java.util.List;

import net.sf.parser4j.kernelgenerator.entity.grammarnode.AbstractMularyGrammarNode;
import net.sf.parser4j.parser.entity.EnumNodeType;

/**
 * common to all non terminal node based on concatenation<br>
 * 
 * @author luc peuvrier
 * 
 */
public abstract class AbstractGrammarNodeConcat extends
		AbstractMularyGrammarNode {

	public AbstractGrammarNodeConcat(final EnumNodeType grammarNodeType) {
		super(grammarNodeType);
	}

	public boolean match(final int recognitionPointIndex) {
		assertOperandListSetted();
		final boolean match;
		if (recognitionPointIndex < 0
				|| recognitionPointIndex > operandList.size()) {
			badRecognitionPointIndex(recognitionPointIndex);
			match = false;// for compiler warning
		} else if (recognitionPointIndex == operandList.size()) {
			match = true;
		} else {
			match = false;
		}
		return match;
	}

	public List<OperandGrammarNode> expectedNonTerminal(
			final int recognitionPointIndex) {
		assertOperandListSetted();
		final List<OperandGrammarNode> expectedNonTerminal;
		if (recognitionPointIndex < 0
				|| recognitionPointIndex > operandList.size()) {
			badRecognitionPointIndex(recognitionPointIndex);
			expectedNonTerminal = null;// for compiler warning
		} else if (recognitionPointIndex == operandList.size()) {
			expectedNonTerminal = EMPTY_LIST;
		} else {
			expectedNonTerminal = new ArrayList<OperandGrammarNode>(1);
			expectedNonTerminal.add(operandList.get(recognitionPointIndex));
		}
		return expectedNonTerminal;
	}

	@Override
	public String toDefinitionString() {
		final StringBuilder builder = new StringBuilder();
		builder.append(nonTerminalToString());
		builder.append(" :(concat)");
		int index = 0;
		if (operandList == null) {
			builder.append(" <no sons: bad state error>");
		} else {
			for (OperandGrammarNode grammarNode : operandList) {
				final String name = grammarNode.getNonTerminalName();
				builder.append(' ');
				if (name == null) {
					builder.append("<null name: bad state error>");
				} else {
					builder.append(name);
					if (grammarNode.isWhiteSpace()) {
						builder.append(" (as white space)");
					}
				}
				if (index < operandList.size() - 1) {
					builder.append(" &");
				}
				index++;
			}
		}
		builder.append(matchClassToString());
		return builder.toString();
	}

	@Override
	public String toString() {
		final StringBuilder builder = new StringBuilder();
		builder.append(super.toString());
		builder.append(" :");
		int index = 0;
		if (operandList == null) {
			builder.append(" <no sons: bad state error>");
		} else {
			for (OperandGrammarNode grammarNode : operandList) {
				builder.append(" NT #");
				builder.append(grammarNode.getNonTerminalIdentifier());
				final String name = grammarNode.getNonTerminalName();
				builder.append(' ');
				if (name == null) {
					builder.append("<null name: bad state error>");
				} else {
					builder.append(name);
					if (grammarNode.isWhiteSpace()) {
						builder.append(" (as white space)");
					}
				}
				if (index < operandList.size() - 1) {
					builder.append(" &");
				}
				index++;
			}
		}
		builder.append(matchClassToString());
		return builder.toString();
	}
}
