/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.entity.grammarnode.impl;

import java.util.List;

import net.sf.parser4j.kernelgenerator.entity.grammarnode.AbstractMularyGrammarNode;
import net.sf.parser4j.parser.entity.EnumNodeType;

/**
 * alternative grammar node
 * 
 * @author luc peuvrier
 * 
 */
public class GrammarNodeAlternative extends AbstractMularyGrammarNode {

	public GrammarNodeAlternative() {
		super(EnumNodeType.ALTERNATIVE);
	}

	public boolean match(final int recognizePointIndex) {
		final boolean match;
		if (recognizePointIndex == 0) {
			match = false;
		} else if (recognizePointIndex == 1) {
			match = true;
		} else {
			badRecognitionPointIndex(recognizePointIndex);
			match = false;// for compiler warning
		}
		return match;
	}

	public List<OperandGrammarNode> expectedNonTerminal(
			final int recognizePointIndex) {
		assertOperandListSetted();
		final List<OperandGrammarNode> expectedNonTerminal;
		if (recognizePointIndex == 0) {
			expectedNonTerminal = operandList;
		} else if (recognizePointIndex == 1) {
			expectedNonTerminal = EMPTY_LIST;
		} else {
			badRecognitionPointIndex(recognizePointIndex);
			expectedNonTerminal = null;// for compiler warning
		}
		return expectedNonTerminal;
	}

	@Override
	public String toDefinitionString() {
		final StringBuilder builder = new StringBuilder();
		builder.append(nonTerminalToString());
		builder.append(" :(alternative)");
		int index = 0;
		if (operandList == null) {
			builder.append(" <no sons: bad state error>");
		} else {
			for (OperandGrammarNode grammarNode : operandList) {
				final String name = grammarNode.getNonTerminalName();
				builder.append(' ');
				if (name == null) {
					builder.append("null name: bad state error");
				} else {
					builder.append(name);
					if (grammarNode.isWhiteSpace()) {
						builder.append(" (as white space)");
					}
				}
				if (index < operandList.size() - 1) {
					builder.append(" |");
				}
				index++;
			}
		}
		builder.append(matchClassToString());
		return builder.toString();
	}

	@Override
	public String toString() {
		final StringBuilder builder = new StringBuilder();
		builder.append(super.toString());
		builder.append(" :");
		int index = 0;
		if (operandList == null) {
			builder.append(" <no sons: bad state error>");
		} else {
			for (OperandGrammarNode grammarNode : operandList) {
				builder.append(" NT #");
				builder.append(grammarNode.getNonTerminalIdentifier());
				final String name = grammarNode.getNonTerminalName();
				builder.append(' ');
				if (name == null) {
					builder.append("<null name: bad state error>");
				} else {
					builder.append(name);
					if (grammarNode.isWhiteSpace()) {
						builder.append(" (as white space)");
					}
				}
				if (index < operandList.size() - 1) {
					builder.append(" |");
				}
				index++;
			}
		}
		builder.append(matchClassToString());
		return builder.toString();
	}
}
