/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.entity.grammarnode.impl;

import java.util.List;

import net.sf.parser4j.kernelgenerator.entity.grammarnode.AbstractNonTerminalNode;
import net.sf.parser4j.kernelgenerator.service.GrammarNodeVisitException;
import net.sf.parser4j.kernelgenerator.service.grammarnode.IGrammarNodeVisitor;
import net.sf.parser4j.parser.entity.EnumNodeType;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class GrammarNodeEmpty extends AbstractNonTerminalNode {

	private static final int[] EMPTY_RIGHT_PART = new int[0];

	public GrammarNodeEmpty() {
		super(EnumNodeType.EMPTY);
	}

	@Override
	public void setOperandList(final List<OperandGrammarNode> operandList) {
		throw new UnsupportedOperationException(getClass() + " have not son");
	}

	public List<OperandGrammarNode> expectedNonTerminal(
			final int recognitionPointIndex) {
		final List<OperandGrammarNode> expectedNonTerminal;
		if (recognitionPointIndex == 0) {
			expectedNonTerminal = EMPTY_LIST;
		} else {
			badRecognitionPointIndex(recognitionPointIndex);
			expectedNonTerminal = null;// for compiler warning
		}
		return expectedNonTerminal;
	}

	public boolean match(final int recognitionPointIndex) {
		final boolean match;
		if (recognitionPointIndex == 0) {
			match = true;
		} else {
			badRecognitionPointIndex(recognitionPointIndex);
			match = false;// for compiler warning
		}
		return match;
	}

	@Override
	public int[] getRightPart() {
		return EMPTY_RIGHT_PART;// NOPMD
	}

	public void accept(final IGrammarNodeVisitor grammarNodeVisitor)
			throws GrammarNodeVisitException {
		if (grammarNodeVisitor.beginVisit(this)) {
			grammarNodeVisitor.endVisit(this);
		}
	}

	@Override
	public String toDefinitionString() {
		final StringBuilder builder = new StringBuilder();
		builder.append(nonTerminalToString());
		builder.append(" :(empty)");
		builder.append(matchClassToString());
		return builder.toString();
	}

	@Override
	public String toString() {
		final StringBuilder builder = new StringBuilder();
		builder.append(super.toString());
		builder.append(" :");
		builder.append(matchClassToString());
		return builder.toString();
	}
}
