/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.entity.grammarnode.impl;

import java.util.ArrayList;

import net.sf.parser4j.kernelgenerator.service.GeneratorException;
import net.sf.parser4j.kernelgenerator.service.GrammarNodeFactory;
import net.sf.parser4j.parser.entity.EnumNodeType;
import net.sf.parser4j.parser.entity.data.NonTerminalMap;

/**
 * string grammar node
 * 
 * @author luc peuvrier
 * 
 */
public class GrammarNodeString extends AbstractGrammarNodeConcat {

	private final static GrammarNodeFactory grammarNodeFactory = GrammarNodeFactory
			.getInstance();

	@SuppressWarnings("unused")
	private String stringValue;// NOPMD

	public GrammarNodeString() {
		super(EnumNodeType.STRING_TERMINAL);
	}

	public void setStringValue(final String stringValue,
			final NonTerminalMap nonTerminalMap) throws GeneratorException {
		this.stringValue = stringValue;
		final char[] values = stringValue.toCharArray();
		operandList = new ArrayList<OperandGrammarNode>(values.length);
		int index = 0;
		for (char terminalValue : values) {
			final String operandTerminalName;
			if (terminalValue > ' ' && terminalValue < 0x7f) {
				operandTerminalName = nonTerminalName + "_" + index + "_"
						+ terminalValue;
			} else {
				operandTerminalName = nonTerminalName + "_" + index + "_0X"
						+ Integer.toString(terminalValue, 16);
			}
			final OperandGrammarNode grammarNodeTerminalChar = grammarNodeFactory
					.newGrammarNodeTerminalChar(nonTerminalMap,
							operandTerminalName, null/* match class mgr */,
							terminalValue);
			operandList.add(grammarNodeTerminalChar);
			index++;
		}
	}

	// @Override
	// public void accept(final IGrammarNodeVisitor grammarNodeVisitor)
	// throws GrammarNodeVisitException {
	// if (grammarNodeVisitor.beginVisit(this)) {
	// grammarNodeVisitor.endVisit(this);
	// }
	// }

	// @Override
	// public String toString() {
	// final StringBuilder builder = new StringBuilder();
	// builder.append(super.toString());
	// builder.append(" string value '");
	// builder.append(stringValue);
	// builder.append('\'');
	// return builder.toString();
	// }
}
