/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.entity.grammarnode.impl;

import java.util.Iterator;
import java.util.NoSuchElementException;

import net.sf.parser4j.kernelgenerator.entity.grammarnode.AbstractTerminalNode;
import net.sf.parser4j.parser.entity.data.TerminalCharRange;

/**
 * terminal character grammar node
 * 
 * @author luc peuvrier
 * 
 */
public class GrammarNodeTerminalChar extends AbstractTerminalNode {

	private int terminalValue;// NOPMD

	public void setTerminalValue(final int terminalValue) {
		this.terminalValue = terminalValue;
	}

	public Iterator<TerminalCharRange> terminalValuesIterator() {
		return new Iterator<TerminalCharRange>() {

			private boolean hasNext = true;// NOPMD

			public boolean hasNext() {
				return hasNext;
			}

			public TerminalCharRange next() {
				if (!hasNext) {
					throw new NoSuchElementException();
				}
				hasNext = false;
				return new TerminalCharRange(terminalValue, terminalValue);
			}

			public void remove() {
				throw new UnsupportedOperationException();
			}
		};
	}

	@Override
	public String valueToString() {
		return characterValueToString(terminalValue);
	}

	@Override
	public String toDefinitionString() {
		final StringBuilder builder = new StringBuilder();
		builder.append(nonTerminalToString());
		builder.append(" :(terminal char) ");
		builder.append(valueToString());
		builder.append(matchClassToString());
		return builder.toString();
	}

	@Override
	public String toString() {
		final StringBuilder builder = new StringBuilder();
		builder.append(super.toString());
		builder.append(" : ");
		builder.append(characterValueToString(terminalValue));
		builder.append(matchClassToString());
		return builder.toString();
	}
}
