/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the Lesser GNU General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.entity.grammarnode.impl;

import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Set;

import net.sf.parser4j.kernelgenerator.entity.grammarnode.AbstractTerminalNode;
import net.sf.parser4j.kernelgenerator.entity.grammarnode.IGrammarNode;
import net.sf.parser4j.parser.entity.data.TerminalCharRange;

/**
 * terminal terminal node set grammar node
 * 
 * @author luc peuvrier
 * 
 */
public class GrammarNodeTerminalSetOfTerminal extends AbstractTerminalNode {

	private Set<IGrammarNode> setOfTerminalNode; // NOPMD

	public void setSetOfTerminalNode(final Set<IGrammarNode> setOfTerminalNode) {
		for (IGrammarNode grammarNode : setOfTerminalNode) {
			if (!grammarNode.isTerminal()) {
				throw new IllegalArgumentException("set of terminal element "
						+ grammarNode + " is not a terminal");
			}
		}
		this.setOfTerminalNode = setOfTerminalNode;
	}

	private class TerminalValuesIterator implements Iterator<TerminalCharRange> {

		private final Iterator<IGrammarNode> setIterator = // NOPMD
		/**/setOfTerminalNode.iterator();

		private Iterator<TerminalCharRange> terminalValueIterator;// NOPMD

		private boolean hasNext;// NOPMD

		private TerminalCharRange next;// NOPMD

		public TerminalValuesIterator() {
			super();
			setNext();
		}

		public boolean hasNext() {
			return hasNext;
		}

		public TerminalCharRange next() {
			if (!hasNext) {
				throw new NoSuchElementException();
			}
			final TerminalCharRange result = next;
			setNext();
			return result;
		}

		private void setNext() {
			boolean done = false;
			do {
				if (terminalValueIterator != null
						&& terminalValueIterator.hasNext()) {
					next = terminalValueIterator.next();
					hasNext = true;
					done = true;
				} else if (setIterator.hasNext()) {
					final IGrammarNode grammarNode = setIterator.next();
					terminalValueIterator = grammarNode
							.terminalValuesIterator();
				} else {
					hasNext = false;
					done = true;
				}
			} while (!done);
		}

		public void remove() {
			throw new UnsupportedOperationException();
		}
	}

	public Iterator<TerminalCharRange> terminalValuesIterator() {
		return new TerminalValuesIterator();
	}

	@Override
	public String valueToString() {
		final StringBuilder builder = new StringBuilder();
		for (IGrammarNode grammarNode : setOfTerminalNode) {
			final AbstractTerminalNode abstractTerminalNode =
			/**/(AbstractTerminalNode) grammarNode;
			builder.append(abstractTerminalNode.valueToString());
		}
		return builder.toString();
	}

	@Override
	public String toDefinitionString() {
		final StringBuilder builder = new StringBuilder();
		builder.append(nonTerminalToString());
		builder.append(" :(set of terminal) [");
		builder.append(valueToString());
		builder.append(']');
		builder.append(matchClassToString());
		return builder.toString();
	}

	@Override
	public String toString() {
		final StringBuilder builder = new StringBuilder();
		builder.append(super.toString());
		builder.append(" : ");
		for (IGrammarNode grammarNode : setOfTerminalNode) {
			builder.append(grammarNode.toString());
			builder.append(' ');
		}
		builder.append(matchClassToString());
		return builder.toString();
	}

}
