/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.service;

import java.util.List;

import net.sf.parser4j.kernelgenerator.entity.grammarnode.IGrammarNode;
import net.sf.parser4j.kernelgenerator.entity.grammarnode.impl.GrammarNodeAlternative;
import net.sf.parser4j.kernelgenerator.entity.grammarnode.impl.GrammarNodeConcat;
import net.sf.parser4j.kernelgenerator.entity.grammarnode.impl.GrammarNodeEmpty;
import net.sf.parser4j.kernelgenerator.entity.grammarnode.impl.GrammarNodeString;
import net.sf.parser4j.kernelgenerator.entity.grammarnode.impl.GrammarNodeTerminalChar;
import net.sf.parser4j.kernelgenerator.entity.grammarnode.impl.OperandGrammarNode;
import net.sf.parser4j.parser.entity.data.NonTerminalMap;

/**
 * 
 * @author luc peuvrier
 * 
 */
public final class GrammarNodeFactory {

	private final static GrammarNodeFactory INSTANCE =
	/**/new GrammarNodeFactory();

	private GrammarNodeFactory() {
		super();
	}

	public static GrammarNodeFactory getInstance() {
		return INSTANCE;
	}

	/**
	 * 
	 * @param nonTerminalMap
	 * @param nonTerminalName
	 *            non terminal name, null if not defined
	 * @param simpleMatchClassName
	 * @param operandList
	 * @return
	 * @throws GeneratorException
	 */
	public IGrammarNode newGrammarNodeAlternative(
			final NonTerminalMap nonTerminalMap, final String nonTerminalName,
			final String simpleMatchClassName,
			final List<OperandGrammarNode> operandList)
			throws GeneratorException {
		final GrammarNodeAlternative grammarNode = new GrammarNodeAlternative();
		grammarNodeSetup(nonTerminalMap, nonTerminalName, simpleMatchClassName,
				grammarNode);
		grammarNode.setOperandList(operandList);
		return grammarNode;
	}

	/**
	 * 
	 * @param nonTerminalMap
	 * @param nonTerminalName
	 *            non terminal name, null if not defined
	 * @param simpleMatchClassName
	 * @param operandList
	 * @return
	 * @throws GeneratorException
	 */
	public IGrammarNode newGrammarNodeConcat(
			final NonTerminalMap nonTerminalMap, final String nonTerminalName,
			final String simpleMatchClassName,
			final List<OperandGrammarNode> operandList)
			throws GeneratorException {
		final GrammarNodeConcat grammarNode = new GrammarNodeConcat();
		grammarNodeSetup(nonTerminalMap, nonTerminalName, simpleMatchClassName,
				grammarNode);
		grammarNode.setOperandList(operandList);
		return grammarNode;
	}

	/**
	 * 
	 * @param nonTerminalMap
	 * @param nonTerminalName
	 *            non terminal name, null if not defined
	 * @param simpleMatchClassName
	 * @param terminalValue
	 * @return
	 * @throws GeneratorException
	 */
	public OperandGrammarNode newGrammarNodeTerminalChar(
			final NonTerminalMap nonTerminalMap, final String nonTerminalName,
			final String simpleMatchClassName, final int terminalValue)
			throws GeneratorException {
		final GrammarNodeTerminalChar grammarNode = new GrammarNodeTerminalChar();
		grammarNodeSetup(nonTerminalMap, nonTerminalName, simpleMatchClassName,
				grammarNode);
		grammarNode.setTerminalValue(terminalValue);
		final OperandGrammarNode operandGrammarNode = new OperandGrammarNode();
		operandGrammarNode.setGrammarNode(grammarNode);
		operandGrammarNode.setWhiteSpace(false);
		return operandGrammarNode;
	}

	/**
	 * 
	 * @param nonTerminalMap
	 * @param nonTerminalName
	 *            non terminal name, null if not defined
	 * @param simpleMatchClassName
	 * @return
	 * @throws GeneratorException
	 */
	public IGrammarNode newGrammarNodeEmpty(
			final NonTerminalMap nonTerminalMap, final String nonTerminalName,
			final String simpleMatchClassName) throws GeneratorException {
		final GrammarNodeEmpty grammarNode = new GrammarNodeEmpty();
		grammarNodeSetup(nonTerminalMap, nonTerminalName, simpleMatchClassName,
				grammarNode);
		return grammarNode;
	}

	/**
	 * 
	 * @param nonTerminalMap
	 * @param nonTerminalName
	 *            non terminal name, null if not defined
	 * @param simpleMatchClassName
	 * @param stringValue
	 * @return
	 * @throws GeneratorException
	 */
	public IGrammarNode newGrammarNodeString(
			final NonTerminalMap nonTerminalMap, final String nonTerminalName,
			final String simpleMatchClassName, final String stringValue)
			throws GeneratorException {
		final GrammarNodeString grammarNode = new GrammarNodeString();
		grammarNodeSetup(nonTerminalMap, nonTerminalName, simpleMatchClassName,
				grammarNode);
		grammarNode.setStringValue(stringValue, nonTerminalMap);
		return grammarNode;
	}

	private void grammarNodeSetup(final NonTerminalMap nonTerminalMap,
			final String nonTerminalName, final String simpleMatchClassName,
			final IGrammarNode grammarNode) throws GeneratorException {
		final boolean token = grammarNode.isToken();
		final int nonTerminalIdentifier = nonTerminalMap
				.createNonTerminalIdentifier(nonTerminalName,
						simpleMatchClassName, token);
		grammarNode.setNonTerminalIdentifier(nonTerminalIdentifier);
		if (nonTerminalName == null) {
			grammarNode.setNonTerminalName(nonTerminalMap.nonTerminalName(
					nonTerminalIdentifier).getName());
		} else {
			grammarNode.setNonTerminalName(nonTerminalName);
		}
	}

}
