/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.service;

import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.xml.XmlBeanFactory;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.FileSystemResource;
import org.springframework.core.io.Resource;

import net.sf.parser4j.kernelgenerator.entity.GrammarTreeFactoryData;
import net.sf.parser4j.kernelgenerator.entity.grammarnode.IGrammarNode;
import net.sf.parser4j.parser.entity.data.NonTerminalMap;

/**
 * grammar factory from spring xml file:<br>
 * must define two beans:<br>
 * <ul>
 * <li>rootGrammarNode</li>
 * <li>nonTerminalMap</li>
 * </ul>
 * 
 * @author luc peuvrier
 * 
 */
public final class GrammarTreeSpringFactory {

	private final static GrammarTreeSpringFactory INSTANCE =
	/**/new GrammarTreeSpringFactory();

	public static GrammarTreeSpringFactory getInstance() {
		return INSTANCE;
	}

	private GrammarTreeSpringFactory() {
		super();
	}

	public GrammarTreeFactoryData createGrammarFromFileSystem(final String path) {
		final Resource resource = new FileSystemResource(path);
		return createGrammar(resource);
	}

	public GrammarTreeFactoryData createGrammarFromClassPath(final String path) {
		final Resource resource = new ClassPathResource(path);
		return createGrammar(resource);
	}

	public GrammarTreeFactoryData createGrammarFromClassPackage(
			final Class<?> clazz, final String resourceName) {
		final String path = clazz.getPackage().getName().replace('.', '/')
				+ "/" + resourceName;
		return createGrammarFromClassPath(path);
	}

	private GrammarTreeFactoryData createGrammar(final Resource resource) {
		final BeanFactory factory = new XmlBeanFactory(resource);
		final IGrammarNode rootGrammarNode = (IGrammarNode) factory
				.getBean("_Start");
		final NonTerminalMap nonTerminalMap = (NonTerminalMap) factory
				.getBean("nonTerminalMap");
		return new GrammarTreeFactoryData(nonTerminalMap, rootGrammarNode);
	}
}
