/*
 * Copyright 2009 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.kernelgenerator.service.grammarnode;

import java.util.Set;
import java.util.TreeSet;

import net.sf.parser4j.kernelgenerator.entity.grammarnode.IGrammarNode;
import net.sf.parser4j.kernelgenerator.service.GrammarNodeVisitException;

/**
 * since grammar is a graph, this is generic to avoid infinite loop.
 * 
 * @author luc peuvrier
 * 
 */
public abstract class AbstractGrammarNodeVisitor implements IGrammarNodeVisitor {

	private transient Set<IGrammarNode> visited;

	protected void initialize() {
		visited = new TreeSet<IGrammarNode>();
	}

	protected void close() {
		visited = null;// NOPMD free memory
	}

	public boolean beginVisit(final IGrammarNode grammarNode)
			throws GrammarNodeVisitException {
		final boolean visit;
		if (visited.add(grammarNode)) {
			beginVisitImpl(grammarNode);
			visit = true;
		} else {
			visitImpl(grammarNode);
			visit = false;
		}
		return visit;
	}

	public void endVisit(final IGrammarNode grammarNode)
			throws GrammarNodeVisitException {
		endVisitImpl(grammarNode);
	}

	protected abstract void beginVisitImpl(final IGrammarNode grammarNode)
			throws GrammarNodeVisitException;

	protected abstract void endVisitImpl(final IGrammarNode grammarNode)
			throws GrammarNodeVisitException;

	protected abstract void visitImpl(final IGrammarNode grammarNode)
			throws GrammarNodeVisitException;
}
