/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.gen4gen.entity;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;

import net.sf.parser4j.parser.service.ParserFileReader;

/**
 * to get reader on grammar input for generator file
 * "base_grammar_input4gen.txt" and "definitive_input_grammar_4gen.txt"
 * 
 * @author luc peuvrier
 * 
 */
public final class InputGrammar4GenReader {

	private static final InputGrammar4GenReader INSTANCE =
	/**/new InputGrammar4GenReader();

	public static InputGrammar4GenReader getInstance() {
		return INSTANCE;
	}

	private InputGrammar4GenReader() {
		super();
	}

	public ParserFileReader createBaseGrammarInput4GenReader()
			throws UnsupportedEncodingException {
		final String inputGrammarFileName = "base_input_grammar_4gen.txt";
		return createReader(inputGrammarFileName);
	}

	public ParserFileReader createDefinitiveGrammarInput4GenReader()
			throws UnsupportedEncodingException {
		final String inputGrammarFileName = "definitive_input_grammar_4gen.txt";
		return createReader(inputGrammarFileName);
	}

	private ParserFileReader createReader(final String inputGrammarFileName)
			throws UnsupportedEncodingException {
		final String path = getClass().getPackage().getName().replace('.', '/')
				+ "/" + inputGrammarFileName;
		final ClassLoader classLoader = // getClass().getClassLoader();
		Thread.currentThread().getContextClassLoader();
		final InputStream inputStream = classLoader.getResourceAsStream(path);
		final InputStreamReader inputStreamReader = new InputStreamReader(
				inputStream, "UTF-8");
		return new ParserFileReader(inputStreamReader, path);
	}
}
