/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.gen4gen.service;

import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;

import net.sf.parser4j.gen4gen.entity.BaseInputGrammar4GenFromSpring;
import net.sf.parser4j.gen4gen.entity.InputGrammar4GenReader;
import net.sf.parser4j.generator.service.ParserDataGenerator;
import net.sf.parser4j.generator.service.ParserDataToHtml;
import net.sf.parser4j.generator.service.match.GenMatchPackage;
import net.sf.parser4j.kernelgenerator.entity.GrammarTreeFactoryData;
import net.sf.parser4j.kernelgenerator.entity.grammarnode.IGrammarNode;
import net.sf.parser4j.kernelgenerator.service.GeneratorException;
import net.sf.parser4j.kernelgenerator.service.KernelParserDataGenerator;
import net.sf.parser4j.parser.entity.ParseResult;
import net.sf.parser4j.parser.entity.data.NonTerminalMap;
import net.sf.parser4j.parser.entity.data.Pair;
import net.sf.parser4j.parser.entity.data.ParserData;
import net.sf.parser4j.parser.service.IParserListener;
import net.sf.parser4j.parser.service.ParserException;
import net.sf.parser4j.parser.service.ParserFileReader;
import net.sf.parser4j.parserdata.ParserDataIOForTest;

import org.apache.log4j.Logger;

/**
 * 
 * @author luc peuvrier
 * 
 */
public final class MainGen4Gen implements IParserListener {

	private static final Logger _log = Logger.getLogger(MainGen4Gen.class);

	private static final KernelParserDataGenerator kernelParserDataGenerator =
	/**/KernelParserDataGenerator.getInstance();

	private static final ParserDataGenerator parserDataGenerator =
	/**/ParserDataGenerator.getInstance();

	private static final InputGrammar4GenReader inputGrammar4GenReader =
	/**/InputGrammar4GenReader.getInstance();

	private static final ParserDataToHtml parserDataToHtml =
	/**/ParserDataToHtml.getInstance();

	private static final ParserDataIOForTest parserDataIOForTest =
	/**/ParserDataIOForTest.getInstance();

	private MainGen4Gen() {
		super();
	}

	public static void main(final String[] args) {
		try {
			final MainGen4Gen mainGen4Gen = new MainGen4Gen();
			mainGen4Gen.run();
		} catch (Exception exception) {
			_log.error("error while generate for generator", exception);
		}
	}

	private void run() throws ParserException, UnsupportedEncodingException,
			GeneratorException, IOException {
		_log.info(
		/**/"get base grammar tree" +
		/**/" from spring xml definition");

		final GrammarTreeFactoryData grammarData =
		/**/BaseInputGrammar4GenFromSpring.getInstance().getGrammar();
		final NonTerminalMap nonTerminalMap = grammarData.getNonTerminalMap();
		final IGrammarNode rootGrammarNode = grammarData.getRootGrammarNode();

		_log.info(
		/**/"generate base parser data from xml grammar");

		final String matchMgrPackageName = GenMatchPackage.getPackageName();
		final ParserData baseXmlParserData = kernelParserDataGenerator
				.generate(rootGrammarNode, nonTerminalMap, matchMgrPackageName);
		parserDataIOForTest.writeBaseXmlParserData(baseXmlParserData);
		File htmlFile = new File("runtime/base_xml_parser_data.html");
		parserDataToHtml.generateHtml(baseXmlParserData, htmlFile);

		_log
				.info("generate base parser data  from base_input_grammar_4gen.txt file");
		ParserFileReader reader = inputGrammar4GenReader
				.createBaseGrammarInput4GenReader();
		File grammarDefTreeTextFile = new File(
				"runtime/base_txt_grammardef.txt");
		File grammarTreeTextFile = new File("runtime/base_txt_grammar.txt");
		Pair<ParseResult, ParserData> pair = parserDataGenerator.generate(
				baseXmlParserData, reader, matchMgrPackageName,
				grammarDefTreeTextFile, grammarTreeTextFile, "ws", this);
		final ParseResult baseTxtParseResult = pair.getMemberA();
		final ParserData baseTxtParserData = pair.getMemberB();
		if (baseTxtParseResult.isInError()) {
			_log.error(baseTxtParseResult.toString());
		} else {

			parserDataIOForTest.writeBaseTxtParseResult(baseTxtParserData);

			htmlFile = new File("runtime/base_txt_parser_data.html");
			parserDataToHtml.generateHtml(baseTxtParserData, htmlFile);

			_log
					.info(
					/**/"generate definitive parser data from definitive_input_grammar_4gen.txt file");

			/* Definitive Grammar Input for Generator Reader */
			reader = inputGrammar4GenReader
					.createDefinitiveGrammarInput4GenReader();
			grammarDefTreeTextFile = new File(
					"runtime/definitive_grammardef.txt");
			grammarTreeTextFile = new File("runtime/definitive_grammar.txt");
			pair = parserDataGenerator.generate(baseTxtParserData, reader,
					matchMgrPackageName, grammarDefTreeTextFile,
					grammarTreeTextFile, null, this);
			final ParseResult definitiveTxtParseResult = pair.getMemberA();
			final ParserData definitiveTxtParserData = pair.getMemberB();
			if (definitiveTxtParseResult.isInError()) {
				_log.error(definitiveTxtParseResult.toString());
			} else {

				/* output parser data to generator parser data package */
				final ParserDataOutputForGenerator parserDataOutputForGenerator =
				/**/ParserDataOutputForGenerator.getInstance();
				parserDataOutputForGenerator
						.writeParserData(definitiveTxtParserData);

				htmlFile = new File("runtime/definitive_parser_data.html");
				parserDataToHtml
						.generateHtml(definitiveTxtParserData, htmlFile);
			}
		}
	}

	@Override
	public void newLine(final int line) {
		// if (_log.isInfoEnabled()) {
		// _log.info("line " + line + " of " + inputFileName);
		// }
	}
}
