/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.entity;

import java.util.Map;
import java.util.TreeMap;

import net.sf.parser4j.parser.service.ParserException;
import net.sf.parser4j.parser.service.ParserFileReader;
import net.sf.parser4j.parser.service.ParserInput;

/**
 * data share between match manager
 * 
 * @author luc peuvrier
 * 
 */
public class ParseSession implements IParseSessionForMatchMgr {

	private transient ParserInput parserInput;

	private transient boolean syntaxMode;

	/** this session attribute map */
	private final transient Map<String, Object> attributeMap =
	/**/new TreeMap<String, Object>();

	/**
	 * @param parserInput
	 *            the parserInput to set
	 */
	public void setParserInput(final ParserInput parserInput) {
		this.parserInput = parserInput;
	}

	/**
	 * @param syntaxMode
	 *            the syntaxMode to set
	 */
	public void setSyntaxMode(final boolean syntaxMode) {
		this.syntaxMode = syntaxMode;
	}

	public void removeAllAttribute() {
		attributeMap.clear();
	}

	public Object getAttribute(final String attributeKey) {
		return attributeMap.get(attributeKey);
	}

	public Object setAttribute(final String attributeKey, final Object attribute) {
		return attributeMap.put(attributeKey, attribute);
	}

	/**
	 * @return parser input current file name
	 * @see net.sf.parser4j.parser.service.ParserInput#getFileName()
	 */
	public String getParserInputFileName() {
		return parserInput.getFileName();
	}

	public void include(final ParserFileReader parserFileReader)
			throws ParserException {
		if (!syntaxMode) {
			throw new ParserException("can not include while parsing token");
		}
		parserInput.include(parserFileReader);
	}
}
