/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.entity;

import java.util.Map;

import net.sf.parser4j.parser.entity.data.NonTerminal;
import net.sf.parser4j.parser.entity.data.NonTerminalMap;
import net.sf.parser4j.parser.entity.parsenode.IParseNode;

/**
 * parse tree information:<br>
 * <ul>
 * <li>{@link IParseNode} the root node</li>{@link NonTerminalMap} non terminal
 * name and identifier number map
 * </ul>
 * used to export in xml file<br>
 * 
 * @author luc peuvrier
 * 
 */
public class ParseTreeInfo {

	private final IParseNode rootNode;// NOPMD

	private final Map<Integer, NonTerminal> nonTerminalByIdentifierMap;// NOPMD

	public ParseTreeInfo(final IParseNode rootNode,
			final Map<Integer, NonTerminal> nonTerminalByIdentifierMap) {
		super();
		this.rootNode = rootNode;
		this.nonTerminalByIdentifierMap = nonTerminalByIdentifierMap;
	}

	public IParseNode getRootNode() {
		return rootNode;
	}

	public Map<Integer, NonTerminal> getNonTerminalByIdentifierMap() {
		return nonTerminalByIdentifierMap;
	}

}
