/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.entity.data;

import java.io.Serializable;
import java.util.Map;
import java.util.NavigableMap;
import java.util.Set;
import java.util.TreeMap;
import java.util.Map.Entry;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class GotoByTerminalEntry implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 3763559404897228094L;

	private final NavigableMap<Integer, TerminalCharRange> lowerEndMap = // NOPMD
	/**/new TreeMap<Integer, TerminalCharRange>();

	private final Map<TerminalCharRange, Integer> gotoMap = // NOPMD
	/**/new TreeMap<TerminalCharRange, Integer>();

	/**
	 * add goto by terminal range
	 * 
	 * @param terminalCharRange
	 * @param toStateIdentifier
	 */
	public void addGoto(final TerminalCharRange terminalCharRange,
			final int toStateIdentifier) {
		lowerEndMap.put(terminalCharRange.getFirstTerminalValue(),
				terminalCharRange);
		gotoMap.put(terminalCharRange, toStateIdentifier);
	}

	/**
	 * 
	 * @param terminal
	 * @return to state by non terminal, -1 if no transition
	 */
	public int toState(final int terminal) {
		final int toState;
		final Entry<Integer, TerminalCharRange> entry = lowerEndMap
				.floorEntry(terminal);
		if (entry == null) {
			toState = -1;
		} else {
			final TerminalCharRange value = entry.getValue();
			if (value.getFirstTerminalValue() <= terminal
					&& value.getLastTerminalValue() >= terminal) {
				toState = gotoMap.get(value);
			} else {
				toState = -1;
			}
		}
		return toState;
	}

	public Set<TerminalCharRange> expected() {
		return gotoMap.keySet();
	}

	public Map<TerminalCharRange, Integer> getGotoMap() {
		return gotoMap;
	}

	@Override
	public String toString() {
		final StringBuilder builder = new StringBuilder();
		for (Map.Entry<TerminalCharRange, Integer> entry : gotoMap.entrySet()) {
			final TerminalCharRange terminalCharRange = entry.getKey();
			builder.append("by (");
			builder.append(terminalCharRange.getFirstTerminalValue());
			builder.append(',');
			builder.append(terminalCharRange.getLastTerminalValue());
			builder.append(") goto ");
			builder.append(entry.getValue());
			builder.append('\n');
		}
		return builder.toString();
	}

}
