/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.entity.data;

import java.io.Serializable;

/**
 * non terminal match entry:<br>
 * <ul>
 * <li>non terminal identifier</li>
 * <li>matched length</li>
 * <li>node type</li>
 * <li>token</li>
 * </ul>
 * 
 * @author luc peuvrier
 * 
 */
public class MatchEntry implements Comparable<MatchEntry>, Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 3954983616230180142L;

	/** matched non terminal */
	private final int matechedNonTerminalIdentifier;// NOPMD

	/** matched length */
	private final int matchedLength;// NOPMD

	/** node type */
	private final int nodeType;// NOPMD

	/** token */
	private final boolean token; // NOPMD

	public MatchEntry(final int nonTerminalIdentifier, final int length,
			final int nodeType, final boolean token) {
		this.matechedNonTerminalIdentifier = nonTerminalIdentifier;
		this.matchedLength = length;
		this.nodeType = nodeType;
		this.token = token;
	}

	public int getMatchedNonTerminalIdentifier() {
		return matechedNonTerminalIdentifier;
	}

	public int getMatchedLength() {
		return matchedLength;
	}

	public int getNodeType() {
		return nodeType;
	}

	public boolean isToken() {
		return token;
	}

	public int compareTo(final MatchEntry matchEntry) {
		return matechedNonTerminalIdentifier
				- matchEntry.getMatchedNonTerminalIdentifier();
	}
}
