/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.entity.data;

import java.util.Map;
import java.util.TreeMap;

import net.sf.parser4j.kernelgenerator.service.GeneratorException;

import org.apache.log4j.Logger;

/**
 * mapping between name and identifier for non terminal
 * 
 * @author luc peuvrier
 * 
 */
public class NonTerminalMap {

	private static final Logger _log = Logger.getLogger(NonTerminalMap.class);

	/** the next free non terminal identifier */
	private int nextFreeNonTerminalIdentifier = 0x10000;// NOPMD

	/** non terminal identifier by non terminal name */
	private final Map<String, Integer> nonTerminalIdentifierByNameMap = // NOPMD
	/**/new TreeMap<String, Integer>();

	/** non terminal creator trace map */
	private final transient Map<String, Exception> creatorTraceMap =
	/**/new TreeMap<String, Exception>();

	/** non terminal data by non terminal identifier */
	private final Map<Integer, NonTerminal> nonTerminalByIdentifierMap = // NOPMD
	/**/new TreeMap<Integer, NonTerminal>();

	/**
	 * non terminal match management class simple name by non terminal
	 * identifier
	 */
	private final Map<Integer, String> nonTerminalMatchClassSimpleNameByIdentifier = // NOPMD
	/**/new TreeMap<Integer, String>();

	public int createNonTerminalIdentifier(
			final String optionnalNonTerminalName, final String matchClassName,
			final boolean token) throws GeneratorException {
		final String nonTerminalName;
		if (optionnalNonTerminalName == null) {
			nonTerminalName = "NT" + (nextFreeNonTerminalIdentifier + 1);
			_log.warn("terminal name have default value",
			/**/new Exception("-trace-"));// NOPMD
		} else {
			if (optionnalNonTerminalName.startsWith("NT")) {
				_log.warn("bad name " + optionnalNonTerminalName
						+ " start with 'NT'",
				/**/new Exception("-trace-"));// NOPMD
			}
			nonTerminalName = optionnalNonTerminalName;
		}
		if (nonTerminalName == null) {
			throw new IllegalArgumentException();
		}
		if (nonTerminalName.startsWith("null")) {
			throw new IllegalArgumentException(nonTerminalName);
		}
		Integer nonTerminalIdentifier = nonTerminalIdentifierByNameMap
				.get(nonTerminalName);
		if (nonTerminalIdentifier == null) {
			creatorTraceMap.put(nonTerminalName, new Exception("last creator"));
			nonTerminalIdentifier = Integer
					.valueOf(nextFreeNonTerminalIdentifier++);
			nonTerminalIdentifierByNameMap.put(nonTerminalName,
					nonTerminalIdentifier);
			final NonTerminal nonTerminal = new NonTerminal(
					nonTerminalIdentifier, nonTerminalName, token);
			nonTerminalByIdentifierMap.put(nonTerminalIdentifier, nonTerminal);
			nonTerminalMatchClassSimpleNameByIdentifier.put(
					nonTerminalIdentifier, matchClassName);
		} else {
			throw new GeneratorException("'" + nonTerminalName
					+ "' already associated to identifier "
					+ nonTerminalIdentifier.intValue(), creatorTraceMap
					.get(nonTerminalName));
		}
		return nonTerminalIdentifier.intValue();
	}

	public int getNonTerminalIdentifier(final String nonTerminalName) {
		return nonTerminalIdentifierByNameMap.get(nonTerminalName);
	}

	public int numberOfNonterminal() {
		return nextFreeNonTerminalIdentifier - 0x10000;
	}

	public NonTerminal nonTerminalName(final int nonTerminalIdentifier) {
		return nonTerminalByIdentifierMap.get(nonTerminalIdentifier);
	}

	public String nonTerminalMatchClassSimpleName(
			final int nonTerminalIdentifier) {
		return nonTerminalMatchClassSimpleNameByIdentifier
				.get(nonTerminalIdentifier);
	}

	public Map<Integer, NonTerminal> getNonTerminalByIdentifierMap() {
		return nonTerminalByIdentifierMap;
	}
}
