/*
 * Copyright 2009 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.entity.data;

import java.io.Serializable;

import net.sf.parser4j.parser.entity.EnumNodeType;

/**
 * recognition point of a basic rule
 * 
 * @author luc peuvrier
 * 
 */
public class ParserDataRecognitionPoint implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 8803168660856705606L;

	private final EnumNodeType nodeType;// NOPMD

	private final int definedNonTerminalIdentifier;// NOPMD

	/** right part of the rule */
	private final int[] rightPart;// NOPMD

	private final int recognitionPointIndex;// NOPMD

	private final boolean token;// NOPMD

	public ParserDataRecognitionPoint(final EnumNodeType nodeType,
			final int definedNonTerminalIdentifier, final int[] rightPart,// NOPMD
			final int recognitionPointIndex, final boolean token) {
		super();
		this.nodeType = nodeType;
		this.definedNonTerminalIdentifier = definedNonTerminalIdentifier;
		this.rightPart = rightPart;
		this.recognitionPointIndex = recognitionPointIndex;
		this.token = token;
	}

	/**
	 * @return the nodeType
	 */
	public EnumNodeType getNodeType() {
		return nodeType;
	}

	/**
	 * @return the definedNonTerminalIdentifier
	 */
	public int getDefinedNonTerminalIdentifier() {
		return definedNonTerminalIdentifier;
	}

	/**
	 * @return the rightPart
	 */
	public int[] getRightPart() {
		return rightPart;// NOPMD
	}

	/**
	 * @return the recognitionPointIndex
	 */
	public int getRecognitionPointIndex() {
		return recognitionPointIndex;
	}

	/**
	 * @return true if token
	 */
	public boolean isToken() {
		return token;
	}

	/**
	 * @return true if terminal
	 * @see net.sf.parser4j.parser.entity.EnumNodeType#isTerminal()
	 */
	public boolean isTerminal() {
		return nodeType.isTerminal();
	}

	/**
	 * @return true if non terminal
	 * @see net.sf.parser4j.parser.entity.EnumNodeType#isNonTerminal()
	 */
	public boolean isNonTerminal() {
		return nodeType.isNonTerminal();
	}

	/**
	 * @return true if concat rule
	 * @see net.sf.parser4j.parser.entity.EnumNodeType#isConcat()
	 */
	public boolean isConcat() {
		return nodeType.isConcat();
	}
}
