/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.entity.data;

import java.io.Serializable;

/**
 * terminal character range: <br>
 * [ {@link #firstTerminalValue} , * {@link #lastTerminalValue} ]<br>
 * 
 * @author luc peuvrier
 * 
 */
public class TerminalCharRange implements Comparable<TerminalCharRange>,
		Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2048555852029234910L;

	private final int firstTerminalValue;// NOPMD

	private final int lastTerminalValue;// NOPMD

	public TerminalCharRange(final int firstTerminalValue,
			final int lastTerminalValue) {
		super();
		if (firstTerminalValue > lastTerminalValue) {
			throw new IllegalArgumentException(
					"first value is greater than last value");
		}
		this.firstTerminalValue = firstTerminalValue;
		this.lastTerminalValue = lastTerminalValue;
	}

	public int getFirstTerminalValue() {
		return firstTerminalValue;
	}

	public int getLastTerminalValue() {
		return lastTerminalValue;
	}

	public int compareTo(final TerminalCharRange terminalCharRange) {
		final int compareTo;
		if (lastTerminalValue < terminalCharRange.firstTerminalValue) {
			compareTo = -1;
		} else if (firstTerminalValue > terminalCharRange
				.getLastTerminalValue()) {
			compareTo = 1;
		} else {
			compareTo = 0;
		}
		return compareTo;
	}

	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + firstTerminalValue;
		result = PRIME * result + lastTerminalValue;
		return result;
	}

	@Override
	@SuppressWarnings("PMD")
	public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		// FIXME use of compareTo()==0 ?
		final TerminalCharRange other = (TerminalCharRange) obj;
		if (firstTerminalValue != other.firstTerminalValue)
			return false;
		if (lastTerminalValue != other.lastTerminalValue)
			return false;
		return true;
	}

	@Override
	public String toString() {
		final StringBuilder stringBuilder = new StringBuilder();
		stringBuilder.append('[');
		if (firstTerminalValue < ' ' || firstTerminalValue >= 0x7f) {
			stringBuilder.append("0X");
			stringBuilder.append(Integer.toString(firstTerminalValue, 16));
		} else {
			stringBuilder.append('\'');
			stringBuilder.append((char) firstTerminalValue);
			stringBuilder.append('\'');
		}
		// FIXME show ascii diplayable character
		/*
		 * if (firstTerminalValue < ' ' && lastTerminalValue < 0x7f) {
		 * stringBuilder.append(" - 0X1f , ' ' - "); } else if
		 * (firstTerminalValue < 0x7f && lastTerminalValue >= 0x7f) {
		 * stringBuilder.append(" ~ , 0X7f -"); } else
		 */{
			stringBuilder.append(" - ");
		}
		if (lastTerminalValue < ' ' || lastTerminalValue >= 0x7f) {
			stringBuilder.append("0X");
			stringBuilder.append(Integer.toString(lastTerminalValue, 16));
		} else {
			stringBuilder.append('\'');
			stringBuilder.append((char) lastTerminalValue);
			stringBuilder.append('\'');
		}

		stringBuilder.append(']');
		return stringBuilder.toString();
	}

}
