/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.entity.parsenode;

import net.sf.parser4j.parser.entity.EnumNodeType;
import net.sf.parser4j.parser.entity.parsenode.data.IParseNodeData;
import net.sf.parser4j.parser.entity.parsenode.data.NullParseNodeData;
import net.sf.parser4j.parser.service.parsenode.IParseNodeVisitor;

/**
 * common to all terminal parse node
 * 
 * @author luc peuvrier
 * 
 */
public abstract class AbstractTerminalParseNode extends AbstractParseNode {

	private IParseNodeData data = NullParseNodeData.getInstance();// NOPMD

	/**
	 * data associated to this parse node by alternative : always one
	 * alternative
	 */
	private IParseNodeData[] dataByAlternative = new IParseNodeData[1];// NOPMD

	protected AbstractTerminalParseNode(final EnumNodeType parseNodeType) {
		super(parseNodeType);
	}

	@Override
	public void accept(final IParseNodeVisitor visitor) {
		if (visitor.beginVisit(this)) {
			visitor.endVisit(this);
		}
	}

	@Override
	public IParseNodeData[] getDataByAlternative() {// NOPMD
		dataByAlternative[0] = data;
		return dataByAlternative;// NOPMD
	}

	@Override
	public int getNumberOfAlternative() {
		return 1;
	}

	@Override
	public IParseNode[][] getSonParseNodesByAlternative() {// NOPMD
		unsupported();
		return null;
	}

	@Override
	public IParseNodeData getUniqData() {
		return data;
	}

	@Override
	public boolean setAlternativeParseNode(
			final IParseNode alternativeParseNode,
			final int maxNumberOfAlternative) {
		// FIXME alternative for terminal !
		// if (!equals(alternativeParseNode)) {
		// throw new IllegalArgumentException(
		// "alternative parse node must be equals to this\n"+toString()+
		// "\nand alternative is\n"+alternativeParseNode.toString());
		// }
		return false;
	}

	@Override
	public void setData(final IParseNodeData data) {
		this.data = data;
	}

	private void unsupported() {
		throw new UnsupportedOperationException("is a terminal parse node");
	}
}
