/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.entity.parsenode;

import net.sf.parser4j.parser.entity.EnumNodeType;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class CharacterParseNode extends AbstractTerminalParseNode {

	private final int terminalValue;// NOPMD

	public CharacterParseNode(final int terminalValue, final int line,
			final int column) {
		super(EnumNodeType.CHARACTER_TERMINAL);
		this.terminalValue = terminalValue;
		setGrammarSymbolIdentifier(terminalValue);
		beginLineNumber = endLineNumber = line;
		beginColumnNumber = endColumnNumber = column;
	}

	public CharacterParseNode(final CharacterParseNode parseNode) {
		super(EnumNodeType.CHARACTER_TERMINAL);
		this.terminalValue = parseNode.terminalValue;
		setGrammarSymbolIdentifier(terminalValue);
		beginLineNumber = parseNode.beginLineNumber;
		endLineNumber = parseNode.endLineNumber;
		beginColumnNumber = parseNode.beginColumnNumber;
		endColumnNumber = parseNode.endColumnNumber;
	}

	public int getTerminalValue() {
		return terminalValue;
	}

	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = super.hashCode();
		result = PRIME * result + terminalValue;
		return result;
	}

	@Override
	@SuppressWarnings("PMD")
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		final CharacterParseNode other = (CharacterParseNode) obj;
		if (terminalValue != other.terminalValue)
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "terminal '" + ((char) terminalValue) + "'\n";
	}
}
