/*
 * Copyright 2009 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.entity.parsenode.status.impl;

import net.sf.parser4j.parser.entity.parsenode.IParseNode;
import net.sf.parser4j.parser.entity.parsenode.status.IParseNodeInErrorStatus;
import net.sf.parser4j.parser.service.DebugUtil;

/**
 * generic for all
 * 
 * @author luc peuvrier
 * 
 */
public abstract class AbstractParseNodeInErrorStatus extends
		AbstractParseNodeStatus implements IParseNodeInErrorStatus {

	private final static DebugUtil debugUtil = DebugUtil.getInstance();

	private final IParseNode inErrorParseNode;// NOPMD

	private final StatusSetterTrace statusSetterTrace; // NOPMD

	private final Exception exception;// NOPMD

	public AbstractParseNodeInErrorStatus(final IParseNode inErrorParseNode,
			final Exception exception, final StatusSetterTrace statusSetterTrace) {
		super(true/* inError */);
		this.inErrorParseNode = inErrorParseNode;
		this.exception = exception;
		this.statusSetterTrace = statusSetterTrace;
	}

	/**
	 * @return the exception
	 */
	public Exception getException() {
		return exception;
	}

	/**
	 * @return the status setter trace
	 */
	public StatusSetterTrace getStatusSetterTrace() {
		return statusSetterTrace;
	}

	/**
	 * @see net.sf.parser4j.parser.entity.parsenode.status.IParseNodeInErrorStatus#getBeginColumnNumber()
	 */
	@Override
	public int getBeginColumnNumber() {
		return inErrorParseNode.getBeginColumnNumber();
	}

	/**
	 * @see net.sf.parser4j.parser.entity.parsenode.status.IParseNodeInErrorStatus#getBeginLineNumber()
	 */
	@Override
	public int getBeginLineNumber() {
		return inErrorParseNode.getBeginLineNumber();
	}

	/**
	 * @see net.sf.parser4j.parser.entity.parsenode.status.IParseNodeInErrorStatus#getEndColumnNumber()
	 */
	@Override
	public int getEndColumnNumber() {
		return inErrorParseNode.getEndColumnNumber();
	}

	/**
	 * @see net.sf.parser4j.parser.entity.parsenode.status.IParseNodeInErrorStatus#getEndLineNumber()
	 */
	@Override
	public int getEndLineNumber() {
		return inErrorParseNode.getEndLineNumber();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		final StringBuilder stringBuilder = new StringBuilder();
		stringBuilder.append(debugUtil.stackTrace(statusSetterTrace));
		if (exception != null) {
			stringBuilder.append("cause by:\n");
			stringBuilder.append(debugUtil.stackTrace(exception));
		}
		stringBuilder.append("\nat ");
		stringBuilder.append(getBeginLineNumber());
		stringBuilder.append(",");
		stringBuilder.append(getBeginColumnNumber());
		stringBuilder.append(" thru ");
		stringBuilder.append(getEndLineNumber());
		stringBuilder.append(",");
		stringBuilder.append(getEndColumnNumber());
		stringBuilder.append(": ");
		stringBuilder.append(getClass().getSimpleName());
		return stringBuilder.toString();
	}
}
