/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.entity.parsestate;

import net.sf.parser4j.parser.service.HasAmbiguityParserException;
import net.sf.parser4j.parser.service.ParserException;

/**
 * parse state is a set of {@link ParseStack}<br>
 * merge when add existing parse stack ( create alternative for ambiguous
 * grammar)
 * 
 * @author luc peuvrier
 * 
 */
public interface IParseState extends Iterable<ParseStack> {

	/**
	 * 
	 * @param parseStack
	 * @return true if added, else is alternative to existing ( ambiguous)
	 * @throws ParserException
	 * @throws HasAmbiguityParserException
	 *             reject ambiguity
	 */
	boolean add(ParseStack parseStack) throws ParserException,
			HasAmbiguityParserException;

	void clear();

	boolean isEmpty();

	int size();

	ParseStack[] toArray();

	/**
	 * 
	 * @return a copy of this parse state
	 */
	IParseState copy();
}
