/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.entity.parsestate;

import java.util.LinkedList;
import java.util.List;

import net.sf.parser4j.parser.service.HasAmbiguityParserException;
import net.sf.parser4j.parser.service.ParserException;

/**
 * specialization for syntax analysis: token recognition management
 * 
 * @author luc peuvrier
 * 
 */
public class SyntaxParseState extends AbstractParseState {

	private List<ParseStack> tokenRecognitionParseStackList = // NOPMD
	/**/new LinkedList<ParseStack>();

	public SyntaxParseState(final boolean acceptAmbiguity,
			final int maxNumberOfAlternative, final boolean[] tokenRecognition) {// NOPMD
		super(acceptAmbiguity, maxNumberOfAlternative, tokenRecognition);
	}

	@Override
	public boolean add(final ParseStack parseStack) throws ParserException,
			HasAmbiguityParserException {
		final boolean added = super.add(parseStack);
		if (added) {
			final int stateNumber = parseStack.getStateNumber();
			if (stateNumber != -1 && tokenRecognition[stateNumber]) {
				final ParseStack tokenParseStack = new ParseStack(stateNumber);
				tokenRecognitionParseStackList.add(tokenParseStack);
			}
		}
		return added;
	}

	public boolean hasTokenRecognition() {
		return true ^ tokenRecognitionParseStackList.isEmpty();
	}

	public List<ParseStack> poolTokenRecognitionParseStackList() {
		final List<ParseStack> result = tokenRecognitionParseStackList;
		tokenRecognitionParseStackList = new LinkedList<ParseStack>();
		return result;
	}
}
