/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.service;

import java.io.IOException;
import java.util.Deque;
import java.util.LinkedList;

import org.apache.log4j.Logger;

/**
 * manage parser input
 * 
 * @author luc peuvrier
 * 
 */
public class ParserInput {

	private static final Logger _log = Logger.getLogger(ParserInput.class);

	private transient final Deque<ParserFileReader> parserFileReaderStack =
	/**/new LinkedList<ParserFileReader>();

	private transient boolean openned;

	public ParserInput(final ParserFileReader parserFileReader)
			throws ParserException {
		super();
		parserFileReaderStack.addFirst(parserFileReader);
		openned = true;
	}

	/**
	 * 
	 * @return character read, or -1 if end of file
	 * @throws IOException
	 */
	public int read() throws IOException {
		if (!openned) {
			throw new IOException("closed");
		}
		int readed;
		try {
			readed = parserFileReaderStack.peekFirst().read();
			while (readed == -1 && parserFileReaderStack.size() > 1) {
				parserFileReaderStack.pollFirst();
				readed = parserFileReaderStack.peekFirst().read();
			}
		} catch (IOException exception) {
			close();
			openned = false;
			throw exception;
		}
		return readed;
	}

	public void close() {
		if (openned) {
			for (ParserFileReader parserFileReader : parserFileReaderStack) {
				try {
					parserFileReader.close();
				} catch (IOException exception) {
					_log.error("closing reader of "
							+ parserFileReader.getFileName(), exception);
				}
			}
			openned = false;
		}
	}

	public String getFileName() {
		return parserFileReaderStack.peekFirst().getFileName();
	}

	public int getColumnNumber() {
		return parserFileReaderStack.peekFirst().getColumnNumber();
	}

	public int getLineNumber() {
		return parserFileReaderStack.peekFirst().getLineNumber();
	}

	/**
	 * Marks the present position in the input. Subsequent calls to
	 * {@link #reset()} will attempt to reposition the input to this point.
	 */
	public void mark() {
		parserFileReaderStack.peekFirst().mark();
	}

	/**
	 * Resets the input to the most recent mark.
	 */
	public void reset() {
		parserFileReaderStack.peekFirst().reset();
	}

	public void include(final ParserFileReader parserFileReader) {
		parserFileReaderStack.addFirst(parserFileReader);
	}
}
