/*
 * Copyright 2009 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.service;

import java.util.Map;

import net.sf.parser4j.parser.entity.data.NonTerminal;
import net.sf.parser4j.parser.entity.parsestate.IParseState;
import net.sf.parser4j.parser.service.parsenode.ParsingToStringUtil;

import org.apache.log4j.Logger;

/**
 * 
 * @author luc peuvrier
 * 
 */
public final class ParserStackLogger {

	private static final Logger _logStack = Logger
			.getLogger(ParserStackLogger.class);

	private static final ParserStackLogger INSTANCE =
	/**/new ParserStackLogger();

	private static final ParsingToStringUtil parsingToStringUtil =
	/**/ParsingToStringUtil.getInstance();

	/** for each state define if it is token recognition */
	private boolean[] tokenRecognition;// NOPMD

	private boolean enabled = true;// NOPMD

	private ParserStackLogger() {
		super();
	}

	public static ParserStackLogger getInstance() {
		return INSTANCE;
	}

	/**
	 * @return
	 * @see org.apache.log4j.Category#isDebugEnabled()
	 */
	public boolean isDebugEnabled() {
		return enabled && _logStack.isDebugEnabled();
	}

	/**
	 * @param nonTerminalByIdentifierMap
	 * @see net.sf.parser4j.parser.service.parsenode.ParsingToStringUtil#setNonTerminalByIdentifierMap(java.util.Map)
	 */
	public void setNonTerminalByIdentifierMap(
			final Map<Integer, NonTerminal> nonTerminalByIdentifierMap) {
		parsingToStringUtil
				.setNonTerminalByIdentifierMap(nonTerminalByIdentifierMap);
	}

	/**
	 * @param tokenRecognition
	 *            the tokenRecognition to set
	 */
	public void setTokenRecognition(final boolean[] tokenRecognition) {// NOPMD
		this.tokenRecognition = tokenRecognition;
	}

	public void setEnabled(final boolean enabled) {
		this.enabled = enabled;
	}

	/**
	 * 
	 * @param title
	 * @param parseState
	 */
	public void logParseState(final String title, final IParseState parseState) {
		final StringBuilder builder = new StringBuilder();
		builder.append(title);
		builder.append('\n');
		if (parseState == null) {
			builder.append("null\n");
		} else {
			builder.append(parsingToStringUtil.parseStateToString(parseState,
					tokenRecognition, true));
		}
		_logStack.debug(builder.toString());
	}

}
