/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.service.match;

import net.sf.parser4j.parser.entity.IParseSessionForMatchMgr;
import net.sf.parser4j.parser.entity.parsenode.IParseNode;
import net.sf.parser4j.parser.service.ParserException;

/**
 * match manager interface
 * 
 * @author luc peuvrier
 * 
 */
public interface IMatchMgr {

	/**
	 * confirm or cancel reduction
	 * 
	 * @param parseSession
	 *            share with all match manager
	 * @param parseNodes
	 *            the sons parse node
	 * @param notWhiteIndexes
	 *            indexes of not white space parse node
	 * @return true if confirm reduction, else false to cancel reduction
	 * @throws ParserException
	 */
	boolean confirmReduction(IParseSessionForMatchMgr parseSession,
			IParseNode[] parseNodes, int[] notWhiteIndexes)
			throws ParserException;

	/**
	 * do the reduction action
	 * 
	 * @param parseSession
	 *            share with all match manager
	 * @param fatherParseNode
	 *            the father parse node, reduction parse node result
	 * @param parseNodes
	 *            the sons parse node
	 * @param notWhiteIndexes
	 *            indexes of not white space parse node
	 * @throws ParserException
	 */
	void reduceAction(IParseSessionForMatchMgr parseSession,
			IParseNode fatherParseNode, IParseNode[] parseNodes,
			int[] notWhiteIndexes) throws ParserException;

}
