/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.service.parsenode;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import net.sf.parser4j.parser.entity.ParseTreeInfo;

import com.thoughtworks.xstream.XStream;

/**
 * read parse tree ( through its root parse node ) from xml in XStream forms
 * 
 * @author luc peuvrier
 * 
 */
public final class ParseTreeFromXml {

	private final static ParseTreeFromXml INSTANCE = new ParseTreeFromXml();

	public static ParseTreeFromXml getInstance() {
		return INSTANCE;
	}

	private ParseTreeFromXml() {
		super();
	}

	public ParseTreeInfo fromXml(final File parseTreeFile)
			throws FileNotFoundException {
		final XStream stream = new XStream();
		FileInputStream inputStream = null;
		final ParseTreeInfo fromXML;
		try {
			inputStream = new FileInputStream(parseTreeFile);
			fromXML = (ParseTreeInfo) stream.fromXML(inputStream);
		} finally {
			if (inputStream != null) {
				try {
					inputStream.close();
				} catch (IOException exception) {// NOPMD
					// ignore error
				}
			}
		}
		return fromXML;
	}
}
