/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parsetreeinspector.entity;

import java.util.ArrayList;
import java.util.List;

import net.sf.parser4j.parser.entity.EnumNodeType;
import net.sf.parser4j.parser.entity.parsenode.CharacterParseNode;
import net.sf.parser4j.parser.entity.parsenode.IParseNode;
import net.sf.parser4j.parser.entity.parsenode.StringParseNode;
import net.sf.parser4j.parser.service.parsenode.ParsingToStringUtil;

/**
 * node for jtree
 * 
 * @author luc peuvrier
 * 
 */
public class JtreeNode implements IJtreeNode {

	private final static ParsingToStringUtil parsingToStringUtil =
	/**/ParsingToStringUtil.getInstance();

	private final List<IJtreeNode> sonList = new ArrayList<IJtreeNode>();// NOPMD

	private final IParseNode parseNode;// NOPMD

	private final boolean alternative;// NOPMD

	private final int alternativeNumber;// NOPMD

	public JtreeNode(final IParseNode parseNode) {
		super();
		this.parseNode = parseNode;
		alternative = false;
		alternativeNumber = 0;
	}

	public JtreeNode(final IParseNode parseNode, final int alternativeNumber) {
		super();
		this.parseNode = parseNode;
		this.alternativeNumber = alternativeNumber;
		alternative = true;
	}

	public IParseNode getParseNode() {
		return parseNode;
	}

	public boolean isAlternative() {
		return alternative;
	}

	public int getAlternativeNumber() {
		return alternativeNumber;
	}

	public void addChild(final IJtreeNode child) {
		sonList.add(child);
	}

	@Override
	public Object getChildAt(final int index) {
		return sonList.get(index);
	}

	@Override
	public int getChildCount() {
		return sonList.size();
	}

	@Override
	public int getIndexOfChild(final IJtreeNode child) {
		return sonList.indexOf(child);
	}

	public int getGrammarSymbolIdentifier() {
		return parseNode.getGrammarSymbolIdentifier();
	}

	public int getMyIdentifier() {
		return parseNode.getMyIdentifier();
	}

	public int getNumberOfAlternative() {
		return parseNode.getNumberOfAlternative();
	}

	public EnumNodeType getParseNodeType() {
		return parseNode.getParseNodeType();
	}

	@Override
	public String toString() {
		final StringBuilder builder = new StringBuilder();
		if (isAlternative()) {
			builder.append("alt #");
			builder.append(getAlternativeNumber());
		} else {
			final EnumNodeType parseNodeType = getParseNodeType();
			builder.append(parseNodeType);
			builder.append(' ');
			final int grammarSymbolIdentifier = getGrammarSymbolIdentifier();
			final String name = parsingToStringUtil
					.grammarSymbolIdentifierToString(grammarSymbolIdentifier);
			builder.append(name);
			if (EnumNodeType.CHARACTER_TERMINAL.equals(parseNodeType)) {

				builder.append(' ');
				final int terminalIdentifier = ((CharacterParseNode) parseNode)
						.getTerminalValue();
				builder.append(parsingToStringUtil
						.terminalIdentifierToString(terminalIdentifier));

			} else if (EnumNodeType.STRING_TERMINAL.equals(parseNodeType)) {

				builder.append(' ');
				builder.append(((StringParseNode) parseNode).getStringValue());
			}
			builder.append(' ');
			builder.append(parseNode.getBeginLineNumber());
			builder.append(',');
			builder.append(parseNode.getBeginColumnNumber());
			builder.append(' ');
			builder.append(parseNode.getEndLineNumber());
			builder.append(',');
			builder.append(parseNode.getEndColumnNumber());
		}
		return builder.toString();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (alternative ? 1231 : 1237);
		result = prime * result + alternativeNumber;
		result = prime * result + parseNode.getMyIdentifier();
		return result;
	}

	@Override
	@SuppressWarnings("PMD")
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		JtreeNode other = (JtreeNode) obj;
		if (alternative != other.alternative)
			return false;
		if (alternativeNumber != other.alternativeNumber)
			return false;
		if (parseNode == null) {
			if (other.parseNode != null)
				return false;
		} else if (parseNode.getMyIdentifier() != other.parseNode
				.getMyIdentifier())
			return false;
		return true;
	}
}
