/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parsetreeinspector.service;

import java.util.Vector;

import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import net.sf.parser4j.parsetreeinspector.entity.IJtreeNode;

/**
 * Jtree model to manage parse tree, tree of {@link IJtreeNode}
 * 
 * @author luc peuvrier
 * 
 */
public class JTreeModelForNode implements TreeModel {

	private final Vector<TreeModelListener> treeModelListeners = // NOPMD
	/**/new Vector<TreeModelListener>();// NOPMD

	private final IJtreeNode rootNode;// NOPMD

	public JTreeModelForNode(final IJtreeNode rootNode) {
		this.rootNode = rootNode;
	}

	public JTreeModelForNode() {
		this.rootNode = null;// NOPMD
	}

	/* Fire events */

	// /**
	// * The only event raised by this model is TreeStructureChanged with the
	// root
	// * as path, i.e. the whole tree has changed.
	// */
	// protected void fireTreeStructureChanged(final IJtreeNode node) {
	// final TreeModelEvent event = new TreeModelEvent(this,
	// new Object[] { node });
	// for (TreeModelListener tml : treeModelListeners) {
	// tml.treeStructureChanged(event);
	// }
	// }
	// protected void fireTreeNodesChanged(final IJtreeNode oldRoot) {
	// final TreeModelEvent event = new TreeModelEvent(this,
	// new Object[] { oldRoot },null,null);
	// for (TreeModelListener tml : treeModelListeners) {
	// tml.treeNodesChanged(event);
	// }
	// }
	/* TreeModel interface implementation */

	public Object getChild(final Object parent, final int index) {
		final IJtreeNode parentNode = (IJtreeNode) parent;
		return parentNode.getChildAt(index);
	}

	public int getChildCount(final Object parent) {
		final IJtreeNode parentNode = (IJtreeNode) parent;
		return parentNode.getChildCount();
	}

	public int getIndexOfChild(final Object parent, final Object child) {
		final IJtreeNode parentNode = (IJtreeNode) parent;
		return parentNode.getIndexOfChild((IJtreeNode) child);
	}

	public Object getRoot() {
		return rootNode;
	}

	public boolean isLeaf(final Object object) {
		final IJtreeNode node = (IJtreeNode) object;
		return node.getChildCount() == 0;
	}

	public void valueForPathChanged(final TreePath path, final Object newValue) {
		// nothing to do
	}

	public void removeTreeModelListener(final TreeModelListener listener) {
		treeModelListeners.removeElement(listener);
	}

	public void addTreeModelListener(final TreeModelListener listener) {
		treeModelListeners.addElement(listener);
	}
}
