/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parsetreeinspector.service;

import java.util.Stack;

import net.sf.parser4j.parser.entity.parsenode.IParseNode;
import net.sf.parser4j.parser.service.parsenode.IParseNodeVisitor;
import net.sf.parser4j.parsetreeinspector.entity.IJtreeNode;
import net.sf.parser4j.parsetreeinspector.entity.JtreeNode;

/**
 * creation of {@link JtreeNode} tree visiting parse tree
 * 
 * @author luc peuvrier
 * 
 */
public class JTreeNodeCreator implements IParseNodeVisitor {

	private IJtreeNode rootNode;// NOPMD

	private final Stack<JtreeNode> stack = new Stack<JtreeNode>();// NOPMD

	public IJtreeNode getRootNode() {
		return rootNode;
	}

	@Override
	public boolean beginVisit(final IParseNode node) {
		final boolean visit;
		final JtreeNode jtreeNode = new JtreeNode(node);
		if (!stack.isEmpty()) {
			final JtreeNode fatherNode = stack.peek();
			fatherNode.addChild(jtreeNode);
		}
		if (stack.contains(jtreeNode)) {
			visit = false;
		} else {
			stack.push(jtreeNode);
			visit = true;
		}
		return visit;
	}

	@Override
	public void endVisit(final IParseNode node) {
		rootNode = stack.pop();
	}

	@Override
	public boolean visitFirstAlternativeOnly() {
		return false;
	}

	@Override
	public void beginAlternativeVisit(final IParseNode node,
			final int alternativeNumber) {
		if (node.getNumberOfAlternative() > 1) {
			final JtreeNode jtreeNode = new JtreeNode(node, alternativeNumber);
			stack.peek().addChild(jtreeNode);
			stack.push(jtreeNode);
		}
	}

	@Override
	public void endAlternativeVisit(final IParseNode node,
			final int alternativeNumber) {
		if (node.getNumberOfAlternative() > 1) {
			stack.pop();
		}
	}
}
