/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parsetreeinspector.service;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.Map;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import net.sf.parser4j.commonfortool.view.filechooser.XmlOpenFileChooser;
import net.sf.parser4j.parser.entity.ParseTreeInfo;
import net.sf.parser4j.parser.entity.data.NonTerminal;
import net.sf.parser4j.parser.entity.parsenode.IParseNode;
import net.sf.parser4j.parser.service.parsenode.ParseTreeFromXml;
import net.sf.parser4j.parser.service.parsenode.ParsingToStringUtil;
import net.sf.parser4j.parsetreeinspector.entity.IJtreeNode;

import org.apache.log4j.Logger;

/**
 * main for parse tree shower
 * 
 * @author luc peuvrier
 * 
 */
public class ShowParseNodeTree extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -2296015048339810387L;

	private static final Logger _log = Logger
			.getLogger(ShowParseNodeTree.class);

	private JFrame frame;// NOPMD

	private File parseTreeFile;// NOPMD

	private JTreeForParseTree tree;// NOPMD

	public static void main(final String[] args) {
		try {
			final ShowParseNodeTree showParseNodeTree =
			/**/new ShowParseNodeTree();
			showParseNodeTree.run();
		} catch (ParseTreeInspectorException exception) {
			_log.error("run error", exception);
		}
	}

	public ShowParseNodeTree() throws ParseTreeInspectorException {
		super(new BorderLayout());
		tree = new JTreeForParseTree();
		final JScrollPane scrollPane = new JScrollPane(tree);
		scrollPane.setPreferredSize(new Dimension(700, 500));
		add(scrollPane, BorderLayout.CENTER);
	}

	public void run() {
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				createAndShowGUI();
				chooseFile();
				try {
					final ParseTreeInfo parseTreeInfo = getRootParseNode();
					final IParseNode rootParseNode = parseTreeInfo
							.getRootNode();
					final Map<Integer, NonTerminal> nonTerminalByIdentifierMap =
					/**/parseTreeInfo.getNonTerminalByIdentifierMap();
					ParsingToStringUtil.getInstance()
							.setNonTerminalByIdentifierMap(
									nonTerminalByIdentifierMap);
					final IJtreeNode rootNode = computeJTree(rootParseNode);
					tree.setRoot(rootNode);
				} catch (ParseTreeInspectorException exception) {
					_log.error("get root parse node error", exception);
					frame.dispose();
				}
			}
		});
	}

	private void createAndShowGUI() {
		frame = new JFrame("Show object tree");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setOpaque(true);
		frame.setContentPane(this);
		frame.pack();
		frame.setVisible(true);
	}

	private void chooseFile() {
		final XmlOpenFileChooser fileChooser = new XmlOpenFileChooser(frame,
				"open parse tree", "runtime");
		if (fileChooser.approve()) {
			parseTreeFile = fileChooser.getSelectedFile();
		} else {
			frame.dispose();
		}
	}

	final private ParseTreeInfo getRootParseNode()
			throws ParseTreeInspectorException {
		final ParseTreeFromXml parseTreeFromXml = ParseTreeFromXml
				.getInstance();
		try {
			return parseTreeFromXml.fromXml(parseTreeFile);
		} catch (FileNotFoundException exception) {
			throw new ParseTreeInspectorException(exception);
		}
	}

	private IJtreeNode computeJTree(final IParseNode rootParseNode) {
		final JTreeNodeCreator treeNodeCreator = new JTreeNodeCreator();
		rootParseNode.accept(treeNodeCreator);
		return treeNodeCreator.getRootNode();
	}
}
