/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.generateandparsetest.test;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.io.Writer;

import org.apache.log4j.Logger;

import net.sf.parser4j.generateandparsetest.grammar.GrammarForTest;
import net.sf.parser4j.generator.service.ParserDataToHtml;
import net.sf.parser4j.kernelgenerator.entity.GrammarTreeFactoryData;
import net.sf.parser4j.kernelgenerator.entity.grammarnode.IGrammarNode;
import net.sf.parser4j.kernelgenerator.service.KernelParserDataGenerator;
import net.sf.parser4j.parser.entity.ParseResult;
import net.sf.parser4j.parser.entity.data.NonTerminalMap;
import net.sf.parser4j.parser.entity.data.ParserData;
import net.sf.parser4j.parser.entity.parsenode.IParseNode;
import net.sf.parser4j.parser.service.HasAmbiguityParserException;
import net.sf.parser4j.parser.service.IParserListener;
import net.sf.parser4j.parser.service.Parser;
import net.sf.parser4j.parser.service.ParserException;
import net.sf.parser4j.parser.service.ParserFileReader;
import net.sf.parser4j.parser.service.parsenode.ParseTreeToXML;
import net.sf.parser4j.parser.service.parsenode.ParsingToStringUtil;
import junit.framework.TestCase;

/**
 * 
 * @author luc peuvrier
 * 
 */
public abstract class AbstractGrammarTest extends TestCase implements
		IParserListener {

	protected final Logger _log = Logger.getLogger(getClass());// NOPMD

	protected final static ParsingToStringUtil parsingToStringUtil = ParsingToStringUtil
			.getInstance();

	protected static final ParseTreeToXML parseNodeToXML = ParseTreeToXML
			.getInstance();

	protected NonTerminalMap nonTerminalMap;// NOPMD

	protected ParserData parserData;// NOPMD

	protected Parser createParser(final String simpleSpringFileName)
			throws ParserException {
		final GrammarTreeFactoryData data = GrammarForTest.getInstance()
				.getGrammar(simpleSpringFileName);
		final IGrammarNode grammar = data.getRootGrammarNode();
		nonTerminalMap = data.getNonTerminalMap();
		parserData = KernelParserDataGenerator.getInstance().generate(grammar,
				nonTerminalMap, "");
		if (_log.isDebugEnabled()) {
			_log.debug(parserData.toString());
		}
		final Parser parser = new Parser(parserData, this);

		parsingToStringUtil.setNonTerminalByIdentifierMap(nonTerminalMap
				.getNonTerminalByIdentifierMap());
		return parser;
	}

	protected ParseResult parse(final Parser parser,
			final boolean syntaxAcceptAmbiguity,
			final boolean lexAcceptAmbiguity, final String input)
			throws ParserException, HasAmbiguityParserException {
		final StringReader reader = new StringReader(input);
		final ParserFileReader parserFileReader =
		/**/new ParserFileReader(reader, "");
		return parser.parse(parserFileReader, syntaxAcceptAmbiguity,
				lexAcceptAmbiguity);
	}

	protected void assertExpected(final String expected,
			final ParseResult result) {
		final IParseNode parseNode = result.getResultParseNode();
		assertEquals("not same output", expected, parsingToStringUtil
				.parseTreeToString(parseNode));
	}

	protected void toXml(final ParseResult result, final String pathname)
			throws ParserException {
		final File outputFile = new File(pathname);
		final IParseNode parseNode = result.getResultParseNode();
		ParseTreeToXML.getInstance().toXml(outputFile, parseNode,
				nonTerminalMap.getNonTerminalByIdentifierMap());
	}

	protected void parseNodeToString(final ParseResult result, final String name)
			throws FileNotFoundException, UnsupportedEncodingException,
			IOException {
		final IParseNode parseNode = result.getResultParseNode();
		final String toString = parsingToStringUtil
				.parseTreeToString(parseNode);
		final FileOutputStream fileOutputStream =
		/**/new FileOutputStream(name);
		final Writer writer = new OutputStreamWriter(fileOutputStream, "UTF8");
		writer.write(toString);
		writer.close();
	}

	protected void toHtml(final String pathname) throws IOException {
		final ParserDataToHtml parserDataToHtml = ParserDataToHtml
				.getInstance();
		final File file = new File(pathname);
		parserDataToHtml.generateHtml(parserData, file);
	}

	public void newLine(final int line) {
		//
	}
}
