/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.generateandparsetest.test;

import java.io.IOException;

import net.sf.parser4j.parser.entity.ParseResult;
import net.sf.parser4j.parser.service.HasAmbiguityParserException;
import net.sf.parser4j.parser.service.Parser;
import net.sf.parser4j.parser.service.ParserException;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class AmbiguousGrammarTest extends AbstractGrammarTest {

	public void testAmbiguousGrammar() throws ParserException, IOException,
			HasAmbiguityParserException {// NOPMD

		final Parser parser = createParser("ambiguous.xml");
		/*
		 * parsing of "a+a*a"
		 */
		final ParseResult result = parse(parser, true, false, "a+a*a");
		assertFalse("must have one result\n" + result, result.isInError());
		assertTrue("must have alternative for ambiguity", parser
				.hasAlternativeForAmbiguity());

		toXml(result, "runtime/parsetree.xml");

		parseNodeToString(result, "runtime/logs/parse_node.txt");

		final String expected =
		/**/"begin ALTERNATIVE Start(65542)\n" +
		/**/" begin ALTERNATIVE Expression(65541)\n" +
		/**/"  begin alternative #0 Expression\n" +
		/**/"   begin CONCAT ExpressionMul(65540)\n" +
		/**/"    begin ALTERNATIVE Expression(65541)\n" + // NOPMD
				/**/"     begin CONCAT ExpressionAdd(65538)\n" +
				/**/"      begin ALTERNATIVE Expression(65541)\n" + // NOPMD
				/**/"       begin CHARACTER_TERMINAL ntA(65536) 'a'\n" +
				/**/"       end ntA\n" +
				/**/"      end Expression\n" +
				/**/"      begin CHARACTER_TERMINAL add(65537) '+'\n" +
				/**/"      end add\n" +
				/**/"      begin ALTERNATIVE Expression(65541)\n" +
				/**/"       begin CHARACTER_TERMINAL ntA(65536) 'a'\n" +
				/**/"       end ntA\n" +
				/**/"      end Expression\n" +
				/**/"     end ExpressionAdd\n" +
				/**/"    end Expression\n" +
				/**/"    begin CHARACTER_TERMINAL mul(65539) '*'\n" +
				/**/"    end mul\n" +
				/**/"    begin ALTERNATIVE Expression(65541)\n" +
				/**/"     begin CHARACTER_TERMINAL ntA(65536) 'a'\n" +
				/**/"     end ntA\n" +
				/**/"    end Expression\n" +
				/**/"   end ExpressionMul\n" +
				/**/"  end alternative #0\n" +
				/**/"  begin alternative #1 Expression\n" +
				/**/"   begin CONCAT ExpressionAdd(65538)\n" +
				/**/"    already visited ALTERNATIVE Expression(65541)\n" +
				/**/"    already visited CHARACTER_TERMINAL add(65537) '+'\n" +
				/**/"    begin ALTERNATIVE Expression(65541)\n" +
				/**/"     begin CONCAT ExpressionMul(65540)\n" +
				/**/"      already visited ALTERNATIVE Expression(65541)\n" +
				/**/"      begin CHARACTER_TERMINAL mul(65539) '*'\n" +
				/**/"      end mul\n" +
				/**/"      begin ALTERNATIVE Expression(65541)\n" +
				/**/"       begin CHARACTER_TERMINAL ntA(65536) 'a'\n" +
				/**/"       end ntA\n" +
				/**/"      end Expression\n" +
				/**/"     end ExpressionMul\n" +
				/**/"    end Expression\n" +
				/**/"   end ExpressionAdd\n" +
				/**/"  end alternative #1\n" +
				/**/" end Expression\n" +
				/**/"end Start\n";

		assertExpected(expected, result);
	}

	public void newLine(final int line) {
		//
	}
}
