/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.generateandparsetest.test;

import java.io.File;
import java.io.IOException;
import java.util.Map;

import junit.framework.TestCase;
import net.sf.parser4j.gen4gen.entity.BaseInputGrammar4GenFromSpring;
import net.sf.parser4j.gen4gen.entity.InputGrammar4GenReader;
import net.sf.parser4j.generator.entity.grammardefnode.IGrammarDefNode;
import net.sf.parser4j.generator.service.GrammarDefNodeToTextFile;
import net.sf.parser4j.generator.service.GrammarDefNodeVisitException;
import net.sf.parser4j.generator.service.match.GenMatchPackage;
import net.sf.parser4j.kernelgenerator.entity.GrammarTreeFactoryData;
import net.sf.parser4j.kernelgenerator.entity.grammarnode.IGrammarNode;
import net.sf.parser4j.kernelgenerator.service.KernelParserDataGenerator;
import net.sf.parser4j.parser.entity.ParseResult;
import net.sf.parser4j.parser.entity.data.NonTerminal;
import net.sf.parser4j.parser.entity.data.NonTerminalMap;
import net.sf.parser4j.parser.entity.data.ParserData;
import net.sf.parser4j.parser.entity.parsenode.IParseNode;
import net.sf.parser4j.parser.service.HasAmbiguityParserException;
import net.sf.parser4j.parser.service.IParserListener;
import net.sf.parser4j.parser.service.Parser;
import net.sf.parser4j.parser.service.ParserException;
import net.sf.parser4j.parser.service.ParserFileReader;
import net.sf.parser4j.parser.service.parsenode.ParseTreeToXML;

import org.apache.log4j.Logger;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class BaseInputGrammarForGenTest extends TestCase implements
		IParserListener {

	private static final Logger _log = Logger
			.getLogger(BaseInputGrammarForGenTest.class);

	private static final ParseTreeToXML parseNodeToXML = ParseTreeToXML
			.getInstance();

	private static final GrammarDefNodeToTextFile grammarDefNodeToTextFile =
	/**/GrammarDefNodeToTextFile.getInstance();

	public void testGeneratorInputGrammar() throws ParserException,
			IOException, GrammarDefNodeVisitException {

		/* get grammar tree from spring xml definition */
		final GrammarTreeFactoryData grammarData =
		/**/BaseInputGrammar4GenFromSpring.getInstance().getGrammar();

		final NonTerminalMap nonTerminalMap = grammarData.getNonTerminalMap();
		final Map<Integer, NonTerminal> nonTerminalByIdentifierMap = nonTerminalMap
				.getNonTerminalByIdentifierMap();
		final IGrammarNode rootGrammarNode = grammarData.getRootGrammarNode();

		/* generate parser data */
		final KernelParserDataGenerator kernelParserDataGenerator =
		/**/KernelParserDataGenerator.getInstance();
		final String matchPackageName = GenMatchPackage.getPackageName();
		final ParserData parserData = kernelParserDataGenerator.generate(
				rootGrammarNode, nonTerminalMap, matchPackageName);
		/* create parser */
		final Parser parser = new Parser(parserData, this);

		/* get base input grammar for generator reader */
		final ParserFileReader reader = InputGrammar4GenReader.getInstance()
				.createBaseGrammarInput4GenReader();
		IParseNode parseNode = null;
		ParseResult result = null;
		try {
			result = parser.parse(reader, false, false);
			parseNode = result.getResultParseNode();
		} catch (HasAmbiguityParserException exception) {
			/*
			 * output information on encountered ambiguity
			 */
			try {
				parseNodeToXML.toXml("runtime/parsetree", exception,
						nonTerminalByIdentifierMap);
			} catch (Exception exception2) {
				_log.error("while output to xml ambiguity", exception2);
			}
			fail("have ambiguity: see 'runtime/parsetree <[0-9]+> .xml'");
		}
		_log.info(result.toString());
		assertNotNull("must have a parsing result", parseNode);

		assertFalse("must not be ambiguous", parser
				.hasAlternativeForAmbiguity());
		final IGrammarDefNode grammarDefNode = (IGrammarDefNode) parseNode
				.getUniqData();
		final File textFile = new File("runtime/grammar_def_node.txt");
		grammarDefNodeToTextFile.generateTextFile(grammarDefNode, textFile);

		// _log.info("output parse tree to \"runtime/ggparsetree.xml\"");
		// final ParsingToStringUtil parsingToStringUtil = ParsingToStringUtil
		// .getInstance();
		// parsingToStringUtil
		// .setNonTerminalByIdentifierMap(nonTerminalByIdentifierMap);
		//
		// final ParseTreeToXML parseNodeToXML = ParseTreeToXML.getInstance();
		// final File outputFile = new File("runtime/ggparsetree.xml");
		// parseNodeToXML.toXml(outputFile, parseNode,
		// nonTerminalByIdentifierMap);
		//
		// _log.info("output parse tree to \"runtime/ggparse_node.txt\"");
		// final String toString = parsingToStringUtil
		// .parseTreeToString(parseNode);
		// final FileOutputStream fileOutputStream =
		// /**/new FileOutputStream("runtime/ggparse_node.txt");
		// final Writer writer = new OutputStreamWriter(fileOutputStream,
		// "UTF8");
		// writer.write(toString);
		// writer.close();
		//
		// _log.info("end output in file");
	}

	public void newLine(final int line) {
		_log.info("line " + line);
	}
}
