/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.generateandparsetest.test;

import java.io.IOException;

import net.sf.parser4j.parser.entity.ParseResult;
import net.sf.parser4j.parser.service.HasAmbiguityParserException;
import net.sf.parser4j.parser.service.Parser;
import net.sf.parser4j.parser.service.ParserException;

/**
 * for token test
 * 
 * @author luc peuvrier
 * 
 */
public class Token2GrammarTest extends AbstractGrammarTest {

	public void test2TokenGrammar() throws ParserException,
			HasAmbiguityParserException, IOException {// NOPMD
		final Parser parser = createParser("id.xml");
		toHtml("runtime/id.html");

		/*
		 * parsing of "ab"
		 */
		final ParseResult result;
		try {
			result = parse(parser, false, false, "ab cd");
		} catch (HasAmbiguityParserException exception) {
			try {
				parseNodeToXML.toXml("runtime/parsetree", exception,
						nonTerminalMap.getNonTerminalByIdentifierMap());
			} catch (Exception exception2) {
				_log.error("while output to xml ambiguity", exception2);
			}
			throw exception;
		}
		assertFalse("must have one result\n" + result, result.isInError());
		assertFalse("must not have alternative for ambiguity", parser
				.hasAlternativeForAmbiguity());

		final String expected =
		/**/"begin ALTERNATIVE Start(65550)\n" +
		/**/" begin CONCAT CatStm(65549)\n" +
		/**/"  begin ALTERNATIVE PrimaryStm(65548)\n" +
		/**/"   begin CONCAT GrammarSymbol(65547)\n" +
		/**/"    begin ALTERNATIVE ws(65538)\n" +
		/**/"     begin EMPTY Empty(65536)\n" +
		/**/"     end Empty\n" +
		/**/"    end ws\n" +
		/**/"    begin ALTERNATIVE Id (token)(65546)\n" +
		/**/"     begin CONCAT Id1(65545)\n" +
		/**/"      begin ALTERNATIVE Id (token)(65546)\n" +
		/**/"       begin CHARACTER_TERMINAL IdAllLetter(65542) 'a'\n" +
		/**/"       end IdAllLetter\n" +
		/**/"      end Id (token)\n" +
		/**/"      begin CHARACTER_TERMINAL AllLetterAndDigit(65544) 'b'\n" +
		/**/"      end AllLetterAndDigit\n" +
		/**/"     end Id1\n" +
		/**/"    end Id (token)\n" +
		/**/"   end GrammarSymbol\n" +
		/**/"  end PrimaryStm\n" +
		/**/"  begin ALTERNATIVE PrimaryStm(65548)\n" +
		/**/"   begin CONCAT GrammarSymbol(65547)\n" +
		/**/"    begin ALTERNATIVE ws(65538)\n" +
		/**/"     begin CHARACTER_TERMINAL Space(65537) ' '\n" +
		/**/"     end Space\n" +
		/**/"    end ws\n" +
		/**/"    begin ALTERNATIVE Id (token)(65546)\n" +
		/**/"     begin CONCAT Id1(65545)\n" +
		/**/"      begin ALTERNATIVE Id (token)(65546)\n" +
		/**/"       begin CHARACTER_TERMINAL IdAllLetter(65542) 'c'\n" +
		/**/"       end IdAllLetter\n" +
		/**/"      end Id (token)\n" +
		/**/"      begin CHARACTER_TERMINAL AllLetterAndDigit(65544) 'd'\n" +
		/**/"      end AllLetterAndDigit\n" +
		/**/"     end Id1\n" +
		/**/"    end Id (token)\n" +
		/**/"   end GrammarSymbol\n" +
		/**/"  end PrimaryStm\n" +
		/**/" end CatStm\n" +
		/**/"end Start\n";

		assertExpected(expected, result);
	}
}
