/*
 * Copyright 2009 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.generator.service;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import net.sf.parser4j.generator.entity.GeneratorOutputFile;
import net.sf.parser4j.grammarfile.GrammarFile;
import net.sf.parser4j.kernelgenerator.service.GeneratorException;
import net.sf.parser4j.parser.entity.ParseResult;
import net.sf.parser4j.parser.service.IParserListener;
import net.sf.parser4j.parser.service.ParserException;

import org.apache.log4j.Logger;

import junit.framework.TestCase;

/**
 * 
 * @author luc peuvrier
 * 
 */
public abstract class AbstractGeneratorTest extends TestCase implements
		IParserListener {

	private final Logger _log = Logger.getLogger(getClass());// NOPMD

	protected static final String MUST_SUCCEED = "must succeed";

	private static final GrammarFile grammarFile =
	/**/GrammarFile.getInstance();

	private static final GeneratorOutputFile generatorOutputFile =
	/**/GeneratorOutputFile.getInstance();

	protected static final ParserDataGenerator parserDataGenerator =
	/**/ParserDataGenerator.getInstance();

	public void testG0() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "g0.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g0_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG1() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "g1.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g1_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG2() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "g2.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g2_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG3() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "g3.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g3_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG4() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "g4.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g4_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG5() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "g5.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g5_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG6() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "g6.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g6_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG7() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "g7.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g7_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG8() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "g8.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g8_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG9() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "g9.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g9_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testGa() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "ga.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"ga_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testGb() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "gb.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"gb_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testGc() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "gc.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"gc_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testGd() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "gd.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"gd_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testGe() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "ge.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"ge_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testGf() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "gf.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"gf_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testGg() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "gg.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"gg_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG1w() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "g1w.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g1w_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG2w() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "g2w.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g2w_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG2w1() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "g2w_1.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g2w_1_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG3w() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "g3w.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g3w_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG4w() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "g4w.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g4w_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG5w() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "g5w.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g5w_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testGtoken() throws GeneratorException, IOException,
			ParserException {
		final String inputGrammarFileName = "gtoken.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"gtoken_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	protected ParseResult generate(final String inputGrammarFileName,
			final String outFileName) throws GeneratorException, IOException,
			ParserException {
		final File inputFile = grammarFile.createFile(inputGrammarFileName);
		final File outFile = new File("runtime/" + outFileName);
		final ParseResult parseResult = generate(inputFile, outFile);
		if (parseResult.isInError()) {
			_log.error(parseResult.toString());
		} else {
			_log.info(parseResult.toString());
			final String out = readFile(outFile);
			final File outExpecteFile = generatorOutputFile
					.createFile(outFileName);
			final String outExpected = readFile(outExpecteFile);
			assertEquals("must be same", outExpected, out);
		}
		return parseResult;
	}

	protected abstract ParseResult generate(final File inputFile,
			final File outFile) throws GeneratorException, ParserException;

	private String readFile(final File file) throws IOException {
		final InputStream inputStream = new FileInputStream(file);
		final StringBuilder stringBuilder = new StringBuilder();
		int readed;
		while ((readed = inputStream.read()) != -1) {// NOPMD
			stringBuilder.append((char) readed);
		}
		inputStream.close();
		return stringBuilder.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sf.parser4j.parser.service.IParserListener#newLine(int)
	 */
	@Override
	public void newLine(final int line) {
		if (_log.isInfoEnabled()) {
			_log.info("line " + line);
		}
	}
}
