/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.generator.service;

import java.io.UnsupportedEncodingException;

import junit.framework.TestCase;
import net.sf.parser4j.gen4gen.entity.InputGrammar4GenReader;
import net.sf.parser4j.generator.entity.parserdata.ParserDataForGeneratorInput;
import net.sf.parser4j.kernelgenerator.service.GeneratorException;
import net.sf.parser4j.parser.entity.ParseResult;
import net.sf.parser4j.parser.entity.data.ParserData;
import net.sf.parser4j.parser.service.HasAmbiguityParserException;
import net.sf.parser4j.parser.service.IParserListener;
import net.sf.parser4j.parser.service.Parser;
import net.sf.parser4j.parser.service.ParserException;
import net.sf.parser4j.parser.service.ParserFileReader;

import org.apache.log4j.Logger;

/**
 * final test: parse definitive generator input grammar text file using parser
 * data generated using this file
 * 
 * @author luc peuvrier
 * 
 */
public class ParseDefinitiveByDefinitiveTest extends TestCase {

	private static final Logger _log = Logger
			.getLogger(ParseDefinitiveByDefinitiveTest.class);

	private static final InputGrammar4GenReader inputGrammar4GenReader =
	/**/InputGrammar4GenReader.getInstance();

	private final static ParserDataForGeneratorInput parserDataForGeneratorInput =
	/**/ParserDataForGeneratorInput.getInstance();

	/**
	 * parse definitive grammar text file using definitive parser data test
	 * 
	 * @throws GeneratorException
	 * @throws ParserException
	 * @throws UnsupportedEncodingException
	 * @throws HasAmbiguityParserException
	 */
	public void testDefinitiveByDefinitive() throws GeneratorException,
			ParserException, UnsupportedEncodingException,
			HasAmbiguityParserException {
		final IParserListener listener = null;
		final ParserData definitiveTxtParserData = parserDataForGeneratorInput
				.read();
		final Parser parser = new Parser(definitiveTxtParserData, listener);
		final ParserFileReader parserFileReader = inputGrammar4GenReader
				.createDefinitiveGrammarInput4GenReader();
		final ParseResult parseResult = parser.parse(parserFileReader,
				false/* syntaxAcceptAmbiguity */, false/* lexAcceptAmbiguity */);
		// final IParseNode rootParseNode =
		// parseResult.getResultParseNode();
		if (parseResult.isInError()) {
			_log.error(parseResult);
			fail("have parsing error");
		} else {
			_log.info(parseResult);
		}
	}
}
