/*
 * Copyright 2009 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.service;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;

import junit.framework.TestCase;
import net.sf.parser4j.grammarfile.GrammarFile;
import net.sf.parser4j.parser.entity.ParseResult;
import net.sf.parser4j.parser.entity.data.ParserData;

import org.apache.log4j.Logger;

/**
 * to test grammar input file with different parser data
 * 
 * @author luc peuvrier
 * 
 */
public abstract class AbstractGrammarInputTest extends TestCase implements
		IParserListener {

	private final Logger _log = Logger.getLogger(getClass());// NOPMD

	private static final GrammarFile grammarFile =
	/**/GrammarFile.getInstance();

	protected ParserData parserData;// NOPMD

	private Parser parser;// NOPMD

	@Override
	protected void setUp() throws Exception {
		parser = new Parser(parserData, this);
		super.setUp();
	}

	@Override
	public void newLine(final int line) {
		// no implementation
	}

	private void parseTest(final String inputGrammarFileName)
			throws UnsupportedEncodingException, FileNotFoundException,
			ParserException, HasAmbiguityParserException {
		final File file = grammarFile.createFile(inputGrammarFileName);
		final FileInputStream fileInputStream = new FileInputStream(file);
		final InputStreamReader inputStreamReader = new InputStreamReader(
				fileInputStream, "UTF-8");
		final ParserFileReader parserFileReader = new ParserFileReader(
				inputStreamReader, inputGrammarFileName);
		final ParseResult parseResult = parser.parse(parserFileReader,
				false/* syntaxAcceptAmbiguity */, false/* lexAcceptAmbiguity */);
		if (parseResult.isInError()) {
			_log.error(parseResult);
			fail("have parsing error");
		} else {
			_log.info(parseResult);
		}
	}

	public void testG0() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("g0.txt");
	}

	public void testG1() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("g1.txt");
	}

	public void testG1w() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("g1w.txt");
	}

	public void testG2() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("g2.txt");
	}

	public void testG2w() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("g2w.txt");
	}

	public void testG2w1() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("g2w_1.txt");
	}

	public void testG3() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("g3.txt");
	}

	public void testG3w() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("g3w.txt");
	}

	public void testG4() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("g4.txt");
	}

	public void testG4w() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("g4w.txt");
	}

	public void testG5() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("g5.txt");
	}

	public void testG6() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("g6.txt");
	}

	public void testG7() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("g7.txt");
	}

	public void testG8() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("g8.txt");
	}

	public void testG9() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("g9.txt");
	}

	public void testGa() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("ga.txt");
	}

	public void testGb() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("gb.txt");
	}

	public void testGc() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("gc.txt");
	}

	public void testGd() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("gd.txt");
	}

	public void testGe() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("ge.txt");
	}

	public void testGf() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("gf.txt");
	}

	public void testGg() throws UnsupportedEncodingException,// NOPMD
			FileNotFoundException, ParserException, HasAmbiguityParserException {
		parseTest("gg.txt");
	}
}
