/*
 * Copyright 2009 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.service;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.util.Map;

import junit.framework.TestCase;
import net.sf.parser4j.generator.entity.parserdata.ParserDataForGeneratorInput;
import net.sf.parser4j.parser.entity.ParseResult;
import net.sf.parser4j.parser.entity.ToParseReader;
import net.sf.parser4j.parser.entity.data.NonTerminal;
import net.sf.parser4j.parser.entity.data.ParserData;
import net.sf.parser4j.parser.entity.parsenode.IParseNode;
import net.sf.parser4j.parser.service.parsenode.ParseTreeToXML;

import org.apache.log4j.Logger;

/**
 * test parsing using parser data used by generator input grammar definition
 * parser
 * 
 * @author luc peuvrier
 * 
 */
public class GeneratorParserDataTest extends TestCase implements
		IParserListener {

	private static final Logger _log = Logger
			.getLogger(GeneratorParserDataTest.class);

	private static final ParserDataForGeneratorInput parserDataForGeneratorInput =
	/**/ParserDataForGeneratorInput.getInstance();

	private static final ToParseReader toParseReader =
	/**/ToParseReader.getInstance();

	private Parser parser;// NOPMD

	private Map<Integer, NonTerminal> nonTerminalByIdentifierMap;// NOPMD

	@Override
	protected void setUp() throws Exception {
		final ParserData parserData = parserDataForGeneratorInput.read();
		parser = new Parser(parserData, this);
		nonTerminalByIdentifierMap = parserData.getNonTerminalByIdentifierMap();
		super.setUp();
	}

	public void testG1() throws UnsupportedEncodingException, ParserException,
			HasAmbiguityParserException {
		final ParserFileReader reader = toParseReader.createReader("g1.txt");
		final ParseResult parseResult = parser.parse(reader,
				false/* syntaxAcceptAmbiguity */, false/* lexAcceptAmbiguity */);
		if (parseResult.isInError()) {
			_log.error(parseResult.toString());
		} else {
			_log.info(parseResult.toString());
		}
		if (!parseResult.isParseError()) {
			final IParseNode parseNode = parseResult.getResultParseNode();
			ParseTreeToXML.getInstance().toXml(new File("runtime/r.xml"),
					parseNode, nonTerminalByIdentifierMap);
		}
		assertFalse("must succeed", parseResult.isInError());
	}

	@Override
	public void newLine(final int line) {
		// no implementation
	}
}
