/*
 * Copyright 2009 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.service;

import java.io.File;
import java.io.UnsupportedEncodingException;

import junit.framework.TestCase;
import net.sf.parser4j.generator.service.ParserDataGenerator;
import net.sf.parser4j.grammarfile.GrammarFile;
import net.sf.parser4j.kernelgenerator.service.GeneratorException;
import net.sf.parser4j.parser.entity.ParseResult;
import net.sf.parser4j.parser.entity.ToParseReader;
import net.sf.parser4j.parser.entity.data.ParserData;
import net.sf.parser4j.parser.service.data.ParserDataIO;
import net.sf.parser4j.parser.service.match.TestMatchPackage;
import net.sf.parser4j.parser.service.parsenode.ParsingToStringUtil;

import org.apache.log4j.Logger;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class ParserTest extends TestCase implements IParserListener {

	private static final Logger _log = Logger.getLogger(ParserTest.class);

	private final static ParserDataGenerator parserDataGenerator =
	/**/ParserDataGenerator.getInstance();

	private final static ParserDataIO parserDataIO =
	/**/ParserDataIO.getInstance();

	private static final GrammarFile grammarFile =
	/**/GrammarFile.getInstance();

	private static final ToParseReader toParseReader =
	/**/ToParseReader.getInstance();

	private final static ParsingToStringUtil parsingToStringUtil =
	/**/ParsingToStringUtil.getInstance();

	public void testGdIn1() throws UnsupportedEncodingException,// NOPMD
			ParserException, GeneratorException, HasAmbiguityParserException {
		generateAndParse("gd.txt", false/* syntaxAcceptAmbiguity */,
				false/* lexAcceptAmbiguity */, "gd_in1.txt");
	}

	public void testGeIn1() throws UnsupportedEncodingException,// NOPMD
			ParserException, GeneratorException, HasAmbiguityParserException {
		generateAndParse("ge.txt", false/* syntaxAcceptAmbiguity */,
				false/* lexAcceptAmbiguity */, "ge_in1.txt");
	}

	public void testGfIn1() throws UnsupportedEncodingException,// NOPMD
			ParserException, GeneratorException, HasAmbiguityParserException {
		generateAndParse("gf.txt", false/* syntaxAcceptAmbiguity */,
				false/* lexAcceptAmbiguity */, "gf_in1.txt");
	}

	public void testGgIn1() throws UnsupportedEncodingException,// NOPMD
			ParserException, GeneratorException, HasAmbiguityParserException {
		generateAndParse("gg.txt", false/* syntaxAcceptAmbiguity */,
				false/* lexAcceptAmbiguity */, "gg_in1.txt");
	}

	public void testGgIn2() throws UnsupportedEncodingException,// NOPMD
			ParserException, GeneratorException, HasAmbiguityParserException {
		generateAndParse("gg.txt", false/* syntaxAcceptAmbiguity */,
				false/* lexAcceptAmbiguity */, "gg_in2.txt");
	}

	public void testGhIn() throws UnsupportedEncodingException,// NOPMD
			ParserException, GeneratorException, HasAmbiguityParserException {
		generateAndParse("gh.txt", false/* syntaxAcceptAmbiguity */,
				false/* lexAcceptAmbiguity */, "gh_in.txt");
	}

	public void testGiIn() throws UnsupportedEncodingException,// NOPMD
			ParserException, GeneratorException, HasAmbiguityParserException {
		generateAndParse("gi.txt", false/* syntaxAcceptAmbiguity */,
				false/* lexAcceptAmbiguity */, "gi_in.txt");
	}

	private void generateAndParse(final String inputGrammarFileName,
			final boolean syntaxAcceptAmbiguity,
			final boolean lexAcceptAmbiguity, final String toParseFileName)
			throws ParserException, GeneratorException,
			HasAmbiguityParserException, UnsupportedEncodingException {

		final File parserDataOutputFile = new File("runtime/parser.data");
		final File grammarInputFile = grammarFile
				.createFile(inputGrammarFileName);
		final File grammarTreeTextFile = new File("runtime/grammartree.txt");
		final ParserStackLogger parserStackLogger = ParserStackLogger
				.getInstance();
		parserStackLogger.setEnabled(false);
		final File htmlOutputFile = new File("runtime/" + inputGrammarFileName
				+ ".html");
		parserDataGenerator.generate(grammarInputFile, htmlOutputFile,
				parserDataOutputFile,
				TestMatchPackage.getPackageName()/* matchMgrPackageName */,
				null/* grammarDefTreeTextFile */, grammarTreeTextFile, this);
		final ParserData parserData = parserDataIO
				.readFromFile(parserDataOutputFile);
		final Parser parser = new Parser(parserData, this);
		final ParserFileReader parserFileReader = toParseReader
				.createReader(toParseFileName);
		parserStackLogger.setEnabled(true);
		final ParseResult parseResult = parser.parse(parserFileReader,
				syntaxAcceptAmbiguity, lexAcceptAmbiguity);
		if (parseResult.isInError()) {
			_log.error(parseResult);
			fail(parseResult.toString());
		} else if (_log.isInfoEnabled()) {
			_log.info(parseResult);
			_log.info(parsingToStringUtil.parseTreeToString(parseResult
					.getResultParseNode()));
		}
	}

	@Override
	public void newLine(final int line) {
		// no implementation
	}
}
