/*
 * Copyright 2009 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parserdata;

import java.io.File;

import net.sf.parser4j.parser.entity.data.ParserData;
import net.sf.parser4j.parser.service.ParserException;
import net.sf.parser4j.parser.service.data.ParserDataIO;

/**
 * to write parser data for test in test resource file
 * 
 * @author luc peuvrier
 * 
 */
public final class ParserDataIOForTest {

	private static final String BASE_XML_PARSER_DATA = "base_xml_parser.data";

	private static final String BASE_TXT_PARSER_DATA = "base_txt_parser.data";

	private static final ParserDataIO parserDataIO = ParserDataIO.getInstance();

	private static final ParserDataIOForTest INSTANCE =
	/**/new ParserDataIOForTest();

	public static ParserDataIOForTest getInstance() {
		return INSTANCE;
	}

	private ParserDataIOForTest() {
		super();
	}

	public void writeBaseXmlParserData(final ParserData baseXmlParserData)
			throws ParserException {
		writeParserData(baseXmlParserData, BASE_XML_PARSER_DATA);
	}

	public void writeBaseTxtParseResult(final ParserData baseTxtParserData)
			throws ParserException {
		writeParserData(baseTxtParserData, BASE_TXT_PARSER_DATA);
	}

	private void writeParserData(final ParserData parserData,
			final String fileName) throws ParserException {
		final String packageName = getClass().getPackage().getName();
		final String path = "src-test/" + packageName.replace('.', '/') + "/"
				+ fileName;
		parserDataIO.writeToFile(new File(path), parserData);
	}

	public ParserData readBaseXmlParserData() throws ParserException {
		return readParserData(BASE_XML_PARSER_DATA);
	}

	public ParserData readBaseTxtParserData() throws ParserException {
		return readParserData(BASE_TXT_PARSER_DATA);
	}

	private ParserData readParserData(final String fileName)
			throws ParserException {
		final String packageName = getClass().getPackage().getName();
		final String path = "src-test/" + packageName.replace('.', '/') + "/"
				+ fileName;
		return parserDataIO.readFromFile(new File(path));
	}
}
